/****************************************************************************
** Meta object code from reading C++ file 'devicenotificationsinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "devicenotificationsinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'devicenotificationsinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN44OrgKdeKdeconnectDeviceNotificationsInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgKdeKdeconnectDeviceNotificationsInterface::qt_create_metaobjectdata<qt_meta_tag_ZN44OrgKdeKdeconnectDeviceNotificationsInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgKdeKdeconnectDeviceNotificationsInterface",
        "allNotificationsRemoved",
        "",
        "notificationPosted",
        "publicId",
        "notificationRemoved",
        "notificationUpdated",
        "activeNotifications",
        "QDBusPendingReply<QStringList>",
        "sendAction",
        "QDBusPendingReply<>",
        "key",
        "action",
        "sendReply",
        "replyId",
        "message"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'allNotificationsRemoved'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notificationPosted'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'notificationRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'notificationUpdated'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'activeNotifications'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>()>(7, 2, QMC::AccessPublic, 0x80000000 | 8),
        // Slot 'sendAction'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, const QString &)>(9, 2, QMC::AccessPublic, 0x80000000 | 10, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 },
        }}),
        // Slot 'sendReply'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, const QString &)>(13, 2, QMC::AccessPublic, 0x80000000 | 10, {{
            { QMetaType::QString, 14 }, { QMetaType::QString, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgKdeKdeconnectDeviceNotificationsInterface, qt_meta_tag_ZN44OrgKdeKdeconnectDeviceNotificationsInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgKdeKdeconnectDeviceNotificationsInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN44OrgKdeKdeconnectDeviceNotificationsInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN44OrgKdeKdeconnectDeviceNotificationsInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN44OrgKdeKdeconnectDeviceNotificationsInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgKdeKdeconnectDeviceNotificationsInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgKdeKdeconnectDeviceNotificationsInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->allNotificationsRemoved(); break;
        case 1: _t->notificationPosted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->notificationRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->notificationUpdated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: { QDBusPendingReply<QStringList> _r = _t->activeNotifications();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->sendAction((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->sendReply((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceNotificationsInterface::*)()>(_a, &OrgKdeKdeconnectDeviceNotificationsInterface::allNotificationsRemoved, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceNotificationsInterface::*)(const QString & )>(_a, &OrgKdeKdeconnectDeviceNotificationsInterface::notificationPosted, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceNotificationsInterface::*)(const QString & )>(_a, &OrgKdeKdeconnectDeviceNotificationsInterface::notificationRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceNotificationsInterface::*)(const QString & )>(_a, &OrgKdeKdeconnectDeviceNotificationsInterface::notificationUpdated, 3))
            return;
    }
}

const QMetaObject *OrgKdeKdeconnectDeviceNotificationsInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgKdeKdeconnectDeviceNotificationsInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN44OrgKdeKdeconnectDeviceNotificationsInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgKdeKdeconnectDeviceNotificationsInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void OrgKdeKdeconnectDeviceNotificationsInterface::allNotificationsRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OrgKdeKdeconnectDeviceNotificationsInterface::notificationPosted(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OrgKdeKdeconnectDeviceNotificationsInterface::notificationRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OrgKdeKdeconnectDeviceNotificationsInterface::notificationUpdated(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
