// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "plugin_connectivity_report_debug.h"


Q_LOGGING_CATEGORY(KDECONNECT_PLUGIN_CONNECTIVITY_REPORT, "kdeconnect.plugin.connectivity_report", QtWarningMsg)

