/****************************************************************************
** Meta object code from reading C++ file 'notification.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kdeconnect-kde-26.04.3/plugins/notifications/notification.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'notification.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12NotificationE_t {};
} // unnamed namespace

template <> constexpr inline auto Notification::qt_create_metaobjectdata<qt_meta_tag_ZN12NotificationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Notification",
        "D-Bus Interface",
        "org.kde.kdeconnect.device.notifications.notification",
        "dismissRequested",
        "",
        "m_internalId",
        "replyRequested",
        "ready",
        "actionTriggered",
        "key",
        "action",
        "replied",
        "message",
        "dismiss",
        "reply",
        "sendReply",
        "internalId",
        "appName",
        "ticker",
        "title",
        "text",
        "groupName",
        "isConversation",
        "isGroupConversation",
        "iconPath",
        "dismissable",
        "hasIcon",
        "silent",
        "replyId"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dismissRequested'
        QtMocHelpers::SignalData<void(const QString &)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'replyRequested'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ready'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Signal 'actionTriggered'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 }, { QMetaType::QString, 10 },
        }}),
        // Signal 'replied'
        QtMocHelpers::SignalData<void(const QString &)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Slot 'dismiss'
        QtMocHelpers::SlotData<void()>(13, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'reply'
        QtMocHelpers::SlotData<void()>(14, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'sendReply'
        QtMocHelpers::SlotData<void(const QString &)>(15, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'internalId'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'appName'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'ticker'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'text'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'groupName'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'isConversation'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'isGroupConversation'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'iconPath'
        QtMocHelpers::PropertyData<QString>(24, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'dismissable'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'hasIcon'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'silent'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'replyId'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<Notification, qt_meta_tag_ZN12NotificationE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Notification::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12NotificationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12NotificationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12NotificationE_t>.metaTypes,
    nullptr
} };

void Notification::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Notification *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dismissRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->replyRequested(); break;
        case 2: _t->ready(); break;
        case 3: _t->actionTriggered((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->replied((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->dismiss(); break;
        case 6: _t->reply(); break;
        case 7: _t->sendReply((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Notification::*)(const QString & )>(_a, &Notification::dismissRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notification::*)()>(_a, &Notification::replyRequested, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notification::*)()>(_a, &Notification::ready, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notification::*)(const QString & , const QString & )>(_a, &Notification::actionTriggered, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Notification::*)(const QString & )>(_a, &Notification::replied, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->internalId(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->appName(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->ticker(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->text(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->groupName(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isConversation(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isGroupConversation(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->iconPath(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->dismissable(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->hasIcon(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->silent(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->replyId(); break;
        default: break;
        }
    }
}

const QMetaObject *Notification::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Notification::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12NotificationE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Notification::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void Notification::dismissRequested(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Notification::replyRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Notification::ready()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Notification::actionTriggered(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void Notification::replied(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
