/****************************************************************************
** Meta object code from reading C++ file 'conversationmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kdeconnect-kde-26.04.3/smsapp/conversationmodel.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'conversationmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17ConversationModelE_t {};
} // unnamed namespace

template <> constexpr inline auto ConversationModel::qt_create_metaobjectdata<qt_meta_tag_ZN17ConversationModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ConversationModel",
        "gotNewMessage",
        "",
        "filePathReceived",
        "filePath",
        "fileName",
        "deviceIdChanged",
        "value",
        "handleConversationUpdate",
        "QDBusVariant",
        "message",
        "handleConversationCreated",
        "sendReplyToConversation",
        "textMessage",
        "QList<QUrl>",
        "attachmentUrls",
        "startNewConversation",
        "QList<ConversationAddress>",
        "addressList",
        "getCharCountInfo",
        "requestAttachmentPath",
        "partID",
        "UniqueIdentifier",
        "requestMessages",
        "ConversationModel::RequestMessageRole",
        "role",
        "threadId",
        "deviceId",
        "Roles",
        "FromMeRole",
        "SenderRole",
        "DateRole",
        "AvatarRole",
        "AttachmentsRole",
        "RequestMessageRole",
        "FetchMore",
        "UiPrefetch",
        "UiTimer"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'gotNewMessage'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'filePathReceived'
        QtMocHelpers::SignalData<void(QString, QString)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'deviceIdChanged'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'handleConversationUpdate'
        QtMocHelpers::SlotData<void(const QDBusVariant &)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'handleConversationCreated'
        QtMocHelpers::SlotData<void(const QDBusVariant &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'sendReplyToConversation'
        QtMocHelpers::MethodData<bool(const QString &, QList<QUrl>)>(12, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Method 'startNewConversation'
        QtMocHelpers::MethodData<bool(const QString &, const QList<ConversationAddress> &, QList<QUrl>)>(16, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 17, 18 }, { 0x80000000 | 14, 15 },
        }}),
        // Method 'getCharCountInfo'
        QtMocHelpers::MethodData<QString(const QString &) const>(19, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 10 },
        }}),
        // Method 'requestAttachmentPath'
        QtMocHelpers::MethodData<void(const qint64 &, const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 21 }, { QMetaType::QString, 22 },
        }}),
        // Method 'requestMessages'
        QtMocHelpers::MethodData<void(ConversationModel::RequestMessageRole)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'threadId'
        QtMocHelpers::PropertyData<qint64>(26, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'deviceId'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'addressList'
        QtMocHelpers::PropertyData<QList<ConversationAddress>>(18, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Roles'
        QtMocHelpers::EnumData<enum Roles>(28, 28, QMC::EnumFlags{}).add({
            {   29, Roles::FromMeRole },
            {   30, Roles::SenderRole },
            {   31, Roles::DateRole },
            {   32, Roles::AvatarRole },
            {   33, Roles::AttachmentsRole },
        }),
        // enum 'RequestMessageRole'
        QtMocHelpers::EnumData<enum RequestMessageRole>(34, 34, QMC::EnumIsScoped).add({
            {   35, RequestMessageRole::FetchMore },
            {   36, RequestMessageRole::UiPrefetch },
            {   37, RequestMessageRole::UiTimer },
        }),
    };
    return QtMocHelpers::metaObjectData<ConversationModel, qt_meta_tag_ZN17ConversationModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ConversationModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QStandardItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17ConversationModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17ConversationModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17ConversationModelE_t>.metaTypes,
    nullptr
} };

void ConversationModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConversationModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->gotNewMessage(); break;
        case 1: _t->filePathReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->deviceIdChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->handleConversationUpdate((*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[1]))); break;
        case 4: _t->handleConversationCreated((*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[1]))); break;
        case 5: { bool _r = _t->sendReplyToConversation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->startNewConversation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<ConversationAddress>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { QString _r = _t->getCharCountInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->requestAttachmentPath((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->requestMessages((*reinterpret_cast<std::add_pointer_t<ConversationModel::RequestMessageRole>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusVariant >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusVariant >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<ConversationAddress> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ConversationModel::*)()>(_a, &ConversationModel::gotNewMessage, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConversationModel::*)(QString , QString )>(_a, &ConversationModel::filePathReceived, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConversationModel::*)(const QString & )>(_a, &ConversationModel::deviceIdChanged, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<ConversationAddress> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qint64*>(_v) = _t->threadId(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->deviceId(); break;
        case 2: *reinterpret_cast<QList<ConversationAddress>*>(_v) = _t->addressList(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setThreadId(*reinterpret_cast<qint64*>(_v)); break;
        case 1: _t->setDeviceId(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setAddressList(*reinterpret_cast<QList<ConversationAddress>*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ConversationModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ConversationModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17ConversationModelE_t>.strings))
        return static_cast<void*>(this);
    return QStandardItemModel::qt_metacast(_clname);
}

int ConversationModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStandardItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void ConversationModel::gotNewMessage()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ConversationModel::filePathReceived(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void ConversationModel::deviceIdChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
