// This file was generated by qtwaylandscanner
// source file is pointer-constraints-unstable-v1.xml

#ifndef QT_WAYLAND_POINTER_CONSTRAINTS_UNSTABLE_V1
#define QT_WAYLAND_POINTER_CONSTRAINTS_UNSTABLE_V1

#include "wayland-pointer-constraints-unstable-v1-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {
    class  zwp_pointer_constraints_v1
    {
    public:
        zwp_pointer_constraints_v1(struct ::wl_registry *registry, uint32_t id, int version);
        zwp_pointer_constraints_v1(struct ::zwp_pointer_constraints_v1 *object);
        zwp_pointer_constraints_v1();

        virtual ~zwp_pointer_constraints_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::zwp_pointer_constraints_v1 *object);

        struct ::zwp_pointer_constraints_v1 *object() { return m_zwp_pointer_constraints_v1; }
        const struct ::zwp_pointer_constraints_v1 *object() const { return m_zwp_pointer_constraints_v1; }
        static zwp_pointer_constraints_v1 *fromObject(struct ::zwp_pointer_constraints_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        enum error {
            error_already_constrained = 1, // pointer constraint already requested on that surface
        };

        enum lifetime {
            lifetime_oneshot = 1,
            lifetime_persistent = 2,
        };

        void destroy();
        struct ::zwp_locked_pointer_v1 *lock_pointer(struct ::wl_surface *surface, struct ::wl_pointer *pointer, struct ::wl_region *region, uint32_t lifetime);
        struct ::zwp_confined_pointer_v1 *confine_pointer(struct ::wl_surface *surface, struct ::wl_pointer *pointer, struct ::wl_region *region, uint32_t lifetime);

    private:
        struct ::zwp_pointer_constraints_v1 *m_zwp_pointer_constraints_v1;
    };

    class  zwp_locked_pointer_v1
    {
    public:
        zwp_locked_pointer_v1(struct ::wl_registry *registry, uint32_t id, int version);
        zwp_locked_pointer_v1(struct ::zwp_locked_pointer_v1 *object);
        zwp_locked_pointer_v1();

        virtual ~zwp_locked_pointer_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::zwp_locked_pointer_v1 *object);

        struct ::zwp_locked_pointer_v1 *object() { return m_zwp_locked_pointer_v1; }
        const struct ::zwp_locked_pointer_v1 *object() const { return m_zwp_locked_pointer_v1; }
        static zwp_locked_pointer_v1 *fromObject(struct ::zwp_locked_pointer_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        void destroy();
        void set_cursor_position_hint(wl_fixed_t surface_x, wl_fixed_t surface_y);
        void set_region(struct ::wl_region *region);

    protected:
        virtual void zwp_locked_pointer_v1_locked();
        virtual void zwp_locked_pointer_v1_unlocked();

    private:
        void init_listener();
        static const struct zwp_locked_pointer_v1_listener m_zwp_locked_pointer_v1_listener;
        static void handle_locked(
            void *data,
            struct ::zwp_locked_pointer_v1 *object);
        static void handle_unlocked(
            void *data,
            struct ::zwp_locked_pointer_v1 *object);
        struct ::zwp_locked_pointer_v1 *m_zwp_locked_pointer_v1;
    };

    class  zwp_confined_pointer_v1
    {
    public:
        zwp_confined_pointer_v1(struct ::wl_registry *registry, uint32_t id, int version);
        zwp_confined_pointer_v1(struct ::zwp_confined_pointer_v1 *object);
        zwp_confined_pointer_v1();

        virtual ~zwp_confined_pointer_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::zwp_confined_pointer_v1 *object);

        struct ::zwp_confined_pointer_v1 *object() { return m_zwp_confined_pointer_v1; }
        const struct ::zwp_confined_pointer_v1 *object() const { return m_zwp_confined_pointer_v1; }
        static zwp_confined_pointer_v1 *fromObject(struct ::zwp_confined_pointer_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        void destroy();
        void set_region(struct ::wl_region *region);

    protected:
        virtual void zwp_confined_pointer_v1_confined();
        virtual void zwp_confined_pointer_v1_unconfined();

    private:
        void init_listener();
        static const struct zwp_confined_pointer_v1_listener m_zwp_confined_pointer_v1_listener;
        static void handle_confined(
            void *data,
            struct ::zwp_confined_pointer_v1 *object);
        static void handle_unconfined(
            void *data,
            struct ::zwp_confined_pointer_v1 *object);
        struct ::zwp_confined_pointer_v1 *m_zwp_confined_pointer_v1;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
