// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "plugin_mousepad_debug.h"


Q_LOGGING_CATEGORY(KDECONNECT_PLUGIN_MOUSEPAD, "kdeconnect.plugin.mousepad", QtWarningMsg)

