// This file was generated by qtwaylandscanner
// source file is relative-pointer-unstable-v1.xml

#include "qwayland-relative-pointer-unstable-v1.h"

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {

static inline void *wlRegistryBind(struct ::wl_registry *registry, uint32_t name, const struct ::wl_interface *interface, uint32_t version)
{
    const uint32_t bindOpCode = 0;
    return (void *) wl_proxy_marshal_constructor_versioned((struct wl_proxy *) registry,
    bindOpCode, interface, version, name, interface->name, version, nullptr);
}

    zwp_relative_pointer_manager_v1::zwp_relative_pointer_manager_v1(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    zwp_relative_pointer_manager_v1::zwp_relative_pointer_manager_v1(struct ::zwp_relative_pointer_manager_v1 *obj)
        : m_zwp_relative_pointer_manager_v1(obj)
    {
    }

    zwp_relative_pointer_manager_v1::zwp_relative_pointer_manager_v1()
        : m_zwp_relative_pointer_manager_v1(nullptr)
    {
    }

    zwp_relative_pointer_manager_v1::~zwp_relative_pointer_manager_v1()
    {
    }

    void zwp_relative_pointer_manager_v1::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_zwp_relative_pointer_manager_v1 = static_cast<struct ::zwp_relative_pointer_manager_v1 *>(wlRegistryBind(registry, id, &zwp_relative_pointer_manager_v1_interface, version));
    }

    void zwp_relative_pointer_manager_v1::init(struct ::zwp_relative_pointer_manager_v1 *obj)
    {
        m_zwp_relative_pointer_manager_v1 = obj;
    }

    zwp_relative_pointer_manager_v1 *zwp_relative_pointer_manager_v1::fromObject(struct ::zwp_relative_pointer_manager_v1 *object)
    {
        return static_cast<zwp_relative_pointer_manager_v1 *>(zwp_relative_pointer_manager_v1_get_user_data(object));
    }

    bool zwp_relative_pointer_manager_v1::isInitialized() const
    {
        return m_zwp_relative_pointer_manager_v1 != nullptr;
    }

    uint32_t zwp_relative_pointer_manager_v1::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_zwp_relative_pointer_manager_v1));
    }

    const struct wl_interface *zwp_relative_pointer_manager_v1::interface()
    {
        return &::zwp_relative_pointer_manager_v1_interface;
    }

    void zwp_relative_pointer_manager_v1::destroy()
    {
        ::zwp_relative_pointer_manager_v1_destroy(
            m_zwp_relative_pointer_manager_v1);
        m_zwp_relative_pointer_manager_v1 = nullptr;
    }

    struct ::zwp_relative_pointer_v1 *zwp_relative_pointer_manager_v1::get_relative_pointer(struct ::wl_pointer *pointer)
    {
        struct ::zwp_relative_pointer_v1 * id = ::zwp_relative_pointer_manager_v1_get_relative_pointer(
            m_zwp_relative_pointer_manager_v1,
            pointer);
        return id;
    }

    zwp_relative_pointer_v1::zwp_relative_pointer_v1(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    zwp_relative_pointer_v1::zwp_relative_pointer_v1(struct ::zwp_relative_pointer_v1 *obj)
        : m_zwp_relative_pointer_v1(obj)
    {
        init_listener();
    }

    zwp_relative_pointer_v1::zwp_relative_pointer_v1()
        : m_zwp_relative_pointer_v1(nullptr)
    {
    }

    zwp_relative_pointer_v1::~zwp_relative_pointer_v1()
    {
    }

    void zwp_relative_pointer_v1::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_zwp_relative_pointer_v1 = static_cast<struct ::zwp_relative_pointer_v1 *>(wlRegistryBind(registry, id, &zwp_relative_pointer_v1_interface, version));
        init_listener();
    }

    void zwp_relative_pointer_v1::init(struct ::zwp_relative_pointer_v1 *obj)
    {
        m_zwp_relative_pointer_v1 = obj;
        init_listener();
    }

    zwp_relative_pointer_v1 *zwp_relative_pointer_v1::fromObject(struct ::zwp_relative_pointer_v1 *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_zwp_relative_pointer_v1_listener)
            return nullptr;
        return static_cast<zwp_relative_pointer_v1 *>(zwp_relative_pointer_v1_get_user_data(object));
    }

    bool zwp_relative_pointer_v1::isInitialized() const
    {
        return m_zwp_relative_pointer_v1 != nullptr;
    }

    uint32_t zwp_relative_pointer_v1::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_zwp_relative_pointer_v1));
    }

    const struct wl_interface *zwp_relative_pointer_v1::interface()
    {
        return &::zwp_relative_pointer_v1_interface;
    }

    void zwp_relative_pointer_v1::destroy()
    {
        ::zwp_relative_pointer_v1_destroy(
            m_zwp_relative_pointer_v1);
        m_zwp_relative_pointer_v1 = nullptr;
    }

    void zwp_relative_pointer_v1::zwp_relative_pointer_v1_relative_motion(uint32_t , uint32_t , wl_fixed_t , wl_fixed_t , wl_fixed_t , wl_fixed_t )
    {
    }

    void zwp_relative_pointer_v1::handle_relative_motion(
        void *data,
        struct ::zwp_relative_pointer_v1 *object,
        uint32_t utime_hi,
        uint32_t utime_lo,
        wl_fixed_t dx,
        wl_fixed_t dy,
        wl_fixed_t dx_unaccel,
        wl_fixed_t dy_unaccel)
    {
        Q_UNUSED(object);
        static_cast<zwp_relative_pointer_v1 *>(data)->zwp_relative_pointer_v1_relative_motion(
            utime_hi,
            utime_lo,
            dx,
            dy,
            dx_unaccel,
            dy_unaccel);
    }

    const struct zwp_relative_pointer_v1_listener zwp_relative_pointer_v1::m_zwp_relative_pointer_v1_listener = {
        zwp_relative_pointer_v1::handle_relative_motion,
    };

    void zwp_relative_pointer_v1::init_listener()
    {
        zwp_relative_pointer_v1_add_listener(m_zwp_relative_pointer_v1, &m_zwp_relative_pointer_v1_listener, this);
    }
}

QT_WARNING_POP
QT_END_NAMESPACE
