/****************************************************************************
** Meta object code from reading C++ file 'smsinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "smsinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smsinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN34OrgKdeKdeconnectDeviceSmsInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgKdeKdeconnectDeviceSmsInterface::qt_create_metaobjectdata<qt_meta_tag_ZN34OrgKdeKdeconnectDeviceSmsInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgKdeKdeconnectDeviceSmsInterface",
        "getAttachment",
        "QDBusPendingReply<>",
        "",
        "partID",
        "uniqueIdentifier",
        "launchApp",
        "requestAllConversations",
        "requestAttachment",
        "requestConversation",
        "conversationID",
        "rangeStartTimestamp",
        "numberToRequest",
        "sendSms",
        "QVariantList",
        "addresses",
        "textMessage",
        "attachmentUrls",
        "subID"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'getAttachment'
        QtMocHelpers::SlotData<QDBusPendingReply<>(qlonglong, const QString &)>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::LongLong, 4 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'launchApp'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(6, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Slot 'requestAllConversations'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(7, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Slot 'requestAttachment'
        QtMocHelpers::SlotData<QDBusPendingReply<>(qlonglong, const QString &)>(8, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::LongLong, 4 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'requestConversation'
        QtMocHelpers::SlotData<QDBusPendingReply<>(qlonglong)>(9, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::LongLong, 10 },
        }}),
        // Slot 'requestConversation'
        QtMocHelpers::SlotData<QDBusPendingReply<>(qlonglong, qlonglong)>(9, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::LongLong, 10 }, { QMetaType::LongLong, 11 },
        }}),
        // Slot 'requestConversation'
        QtMocHelpers::SlotData<QDBusPendingReply<>(qlonglong, qlonglong, qlonglong)>(9, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::LongLong, 10 }, { QMetaType::LongLong, 11 }, { QMetaType::LongLong, 12 },
        }}),
        // Slot 'sendSms'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QVariantList &, const QString &, const QVariantList &)>(13, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { 0x80000000 | 14, 15 }, { QMetaType::QString, 16 }, { 0x80000000 | 14, 17 },
        }}),
        // Slot 'sendSms'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QVariantList &, const QString &, const QVariantList &, qlonglong)>(13, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { 0x80000000 | 14, 15 }, { QMetaType::QString, 16 }, { 0x80000000 | 14, 17 }, { QMetaType::LongLong, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgKdeKdeconnectDeviceSmsInterface, qt_meta_tag_ZN34OrgKdeKdeconnectDeviceSmsInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgKdeKdeconnectDeviceSmsInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34OrgKdeKdeconnectDeviceSmsInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34OrgKdeKdeconnectDeviceSmsInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN34OrgKdeKdeconnectDeviceSmsInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgKdeKdeconnectDeviceSmsInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgKdeKdeconnectDeviceSmsInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDBusPendingReply<> _r = _t->getAttachment((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDBusPendingReply<> _r = _t->launchApp();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<> _r = _t->requestAllConversations();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<> _r = _t->requestAttachment((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<> _r = _t->requestConversation((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->requestConversation((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->requestConversation((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->sendSms((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<> _r = _t->sendSms((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *OrgKdeKdeconnectDeviceSmsInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgKdeKdeconnectDeviceSmsInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34OrgKdeKdeconnectDeviceSmsInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgKdeKdeconnectDeviceSmsInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
