/****************************************************************************
** Meta object code from reading C++ file 'inputcapturesession.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kdeconnect-kde-26.04.3/plugins/shareinputdevices/inputcapturesession.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'inputcapturesession.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19InputCaptureSessionE_t {};
} // unnamed namespace

template <> constexpr inline auto InputCaptureSession::qt_create_metaobjectdata<qt_meta_tag_ZN19InputCaptureSessionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "InputCaptureSession",
        "started",
        "",
        "QLine",
        "barrier",
        "QPointF",
        "delta",
        "mouseMove",
        "dx",
        "dy",
        "mouseButton",
        "button",
        "pressed",
        "scrollDelta",
        "scrollDiscrete",
        "key",
        "Qt::Key",
        "Qt::KeyboardModifiers",
        "modifiers",
        "text"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'started'
        QtMocHelpers::SignalData<void(const QLine &, const QPointF &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'mouseMove'
        QtMocHelpers::SignalData<void(double, double)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 }, { QMetaType::Double, 9 },
        }}),
        // Signal 'mouseButton'
        QtMocHelpers::SignalData<void(int, bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'scrollDelta'
        QtMocHelpers::SignalData<void(double, double)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 8 }, { QMetaType::Double, 9 },
        }}),
        // Signal 'scrollDiscrete'
        QtMocHelpers::SignalData<void(int, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::Int, 9 },
        }}),
        // Signal 'key'
        QtMocHelpers::SignalData<void(Qt::Key, Qt::KeyboardModifiers, const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 15 }, { 0x80000000 | 17, 18 }, { QMetaType::QString, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<InputCaptureSession, qt_meta_tag_ZN19InputCaptureSessionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject InputCaptureSession::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19InputCaptureSessionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19InputCaptureSessionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19InputCaptureSessionE_t>.metaTypes,
    nullptr
} };

void InputCaptureSession::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<InputCaptureSession *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->started((*reinterpret_cast<std::add_pointer_t<QLine>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 1: _t->mouseMove((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 2: _t->mouseButton((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->scrollDelta((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2]))); break;
        case 4: _t->scrollDiscrete((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->key((*reinterpret_cast<std::add_pointer_t<Qt::Key>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::KeyboardModifiers>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (InputCaptureSession::*)(const QLine & , const QPointF & )>(_a, &InputCaptureSession::started, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputCaptureSession::*)(double , double )>(_a, &InputCaptureSession::mouseMove, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputCaptureSession::*)(int , bool )>(_a, &InputCaptureSession::mouseButton, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputCaptureSession::*)(double , double )>(_a, &InputCaptureSession::scrollDelta, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputCaptureSession::*)(int , int )>(_a, &InputCaptureSession::scrollDiscrete, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputCaptureSession::*)(Qt::Key , Qt::KeyboardModifiers , const QString & )>(_a, &InputCaptureSession::key, 5))
            return;
    }
}

const QMetaObject *InputCaptureSession::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *InputCaptureSession::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19InputCaptureSessionE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int InputCaptureSession::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void InputCaptureSession::started(const QLine & _t1, const QPointF & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void InputCaptureSession::mouseMove(double _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void InputCaptureSession::mouseButton(int _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void InputCaptureSession::scrollDelta(double _t1, double _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void InputCaptureSession::scrollDiscrete(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void InputCaptureSession::key(Qt::Key _t1, Qt::KeyboardModifiers _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
