/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.portal.InputCapture.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef INPUTCAPTURE_H
#define INPUTCAPTURE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.freedesktop.portal.InputCapture
 */
class OrgFreedesktopPortalInputCaptureInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.portal.InputCapture"; }

public:
    OrgFreedesktopPortalInputCaptureInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgFreedesktopPortalInputCaptureInterface();

    Q_PROPERTY(uint SupportedCapabilities READ supportedCapabilities)
    inline uint supportedCapabilities() const
    { return qvariant_cast< uint >(property("SupportedCapabilities")); }

    Q_PROPERTY(uint version READ version)
    inline uint version() const
    { return qvariant_cast< uint >(property("version")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<QDBusUnixFileDescriptor> ConnectToEIS(const QDBusObjectPath &session_handle, const QVariantMap &options)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(session_handle) << QVariant::fromValue(options);
        return asyncCallWithArgumentList(QStringLiteral("ConnectToEIS"), argumentList);
    }

    inline QDBusPendingReply<QDBusObjectPath> CreateSession(const QString &parent_window, const QVariantMap &options)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(parent_window) << QVariant::fromValue(options);
        return asyncCallWithArgumentList(QStringLiteral("CreateSession"), argumentList);
    }

    inline QDBusPendingReply<> Disable(const QDBusObjectPath &session_handle, const QVariantMap &options)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(session_handle) << QVariant::fromValue(options);
        return asyncCallWithArgumentList(QStringLiteral("Disable"), argumentList);
    }

    inline QDBusPendingReply<> Enable(const QDBusObjectPath &session_handle, const QVariantMap &options)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(session_handle) << QVariant::fromValue(options);
        return asyncCallWithArgumentList(QStringLiteral("Enable"), argumentList);
    }

    inline QDBusPendingReply<QDBusObjectPath> GetZones(const QDBusObjectPath &session_handle, const QVariantMap &options)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(session_handle) << QVariant::fromValue(options);
        return asyncCallWithArgumentList(QStringLiteral("GetZones"), argumentList);
    }

    inline QDBusPendingReply<> Release(const QDBusObjectPath &session_handle, const QVariantMap &options)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(session_handle) << QVariant::fromValue(options);
        return asyncCallWithArgumentList(QStringLiteral("Release"), argumentList);
    }

    inline QDBusPendingReply<QDBusObjectPath> SetPointerBarriers(const QDBusObjectPath &session_handle, const QVariantMap &options, const QList<QVariantMap> &barriers, uint zone_set)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(session_handle) << QVariant::fromValue(options) << QVariant::fromValue(barriers) << QVariant::fromValue(zone_set);
        return asyncCallWithArgumentList(QStringLiteral("SetPointerBarriers"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void Activated(const QDBusObjectPath &session_handle, const QVariantMap &options);
    void Deactivated(const QDBusObjectPath &session_handle, const QVariantMap &options);
    void Disabled(const QDBusObjectPath &session_handle, const QVariantMap &options);
    void ZonesChanged(const QDBusObjectPath &session_handle, const QVariantMap &options);
};

#endif
// clazy:skip
