/****************************************************************************
** Meta object code from reading C++ file 'inputcapture.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "inputcapture.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'inputcapture.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN41OrgFreedesktopPortalInputCaptureInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopPortalInputCaptureInterface::qt_create_metaobjectdata<qt_meta_tag_ZN41OrgFreedesktopPortalInputCaptureInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopPortalInputCaptureInterface",
        "Activated",
        "",
        "QDBusObjectPath",
        "session_handle",
        "QVariantMap",
        "options",
        "Deactivated",
        "Disabled",
        "ZonesChanged",
        "ConnectToEIS",
        "QDBusPendingReply<QDBusUnixFileDescriptor>",
        "CreateSession",
        "QDBusPendingReply<QDBusObjectPath>",
        "parent_window",
        "Disable",
        "QDBusPendingReply<>",
        "Enable",
        "GetZones",
        "Release",
        "SetPointerBarriers",
        "QList<QVariantMap>",
        "barriers",
        "zone_set",
        "SupportedCapabilities",
        "version"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'Activated'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const QVariantMap &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'Deactivated'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const QVariantMap &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'Disabled'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const QVariantMap &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'ZonesChanged'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const QVariantMap &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'ConnectToEIS'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusUnixFileDescriptor>(const QDBusObjectPath &, const QVariantMap &)>(10, 2, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'CreateSession'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(const QString &, const QVariantMap &)>(12, 2, QMC::AccessPublic, 0x80000000 | 13, {{
            { QMetaType::QString, 14 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'Disable'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &)>(15, 2, QMC::AccessPublic, 0x80000000 | 16, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'Enable'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &)>(17, 2, QMC::AccessPublic, 0x80000000 | 16, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'GetZones'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(const QDBusObjectPath &, const QVariantMap &)>(18, 2, QMC::AccessPublic, 0x80000000 | 13, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'Release'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QDBusObjectPath &, const QVariantMap &)>(19, 2, QMC::AccessPublic, 0x80000000 | 16, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'SetPointerBarriers'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(const QDBusObjectPath &, const QVariantMap &, const QList<QVariantMap> &, uint)>(20, 2, QMC::AccessPublic, 0x80000000 | 13, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 }, { 0x80000000 | 21, 22 }, { QMetaType::UInt, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'SupportedCapabilities'
        QtMocHelpers::PropertyData<uint>(24, QMetaType::UInt, QMC::DefaultPropertyFlags),
        // property 'version'
        QtMocHelpers::PropertyData<uint>(25, QMetaType::UInt, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopPortalInputCaptureInterface, qt_meta_tag_ZN41OrgFreedesktopPortalInputCaptureInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopPortalInputCaptureInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN41OrgFreedesktopPortalInputCaptureInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN41OrgFreedesktopPortalInputCaptureInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN41OrgFreedesktopPortalInputCaptureInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopPortalInputCaptureInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopPortalInputCaptureInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->Activated((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 1: _t->Deactivated((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 2: _t->Disabled((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 3: _t->ZonesChanged((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 4: { QDBusPendingReply<QDBusUnixFileDescriptor> _r = _t->ConnectToEIS((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusUnixFileDescriptor>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<QDBusObjectPath> _r = _t->CreateSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->Disable((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->Enable((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<QDBusObjectPath> _r = _t->GetZones((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<> _r = _t->Release((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<QDBusObjectPath> _r = _t->SetPointerBarriers((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<QVariantMap>>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopPortalInputCaptureInterface::*)(const QDBusObjectPath & , const QVariantMap & )>(_a, &OrgFreedesktopPortalInputCaptureInterface::Activated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopPortalInputCaptureInterface::*)(const QDBusObjectPath & , const QVariantMap & )>(_a, &OrgFreedesktopPortalInputCaptureInterface::Deactivated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopPortalInputCaptureInterface::*)(const QDBusObjectPath & , const QVariantMap & )>(_a, &OrgFreedesktopPortalInputCaptureInterface::Disabled, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopPortalInputCaptureInterface::*)(const QDBusObjectPath & , const QVariantMap & )>(_a, &OrgFreedesktopPortalInputCaptureInterface::ZonesChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uint*>(_v) = _t->supportedCapabilities(); break;
        case 1: *reinterpret_cast<uint*>(_v) = _t->version(); break;
        default: break;
        }
    }
}

const QMetaObject *OrgFreedesktopPortalInputCaptureInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopPortalInputCaptureInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN41OrgFreedesktopPortalInputCaptureInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopPortalInputCaptureInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void OrgFreedesktopPortalInputCaptureInterface::Activated(const QDBusObjectPath & _t1, const QVariantMap & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OrgFreedesktopPortalInputCaptureInterface::Deactivated(const QDBusObjectPath & _t1, const QVariantMap & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void OrgFreedesktopPortalInputCaptureInterface::Disabled(const QDBusObjectPath & _t1, const QVariantMap & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OrgFreedesktopPortalInputCaptureInterface::ZonesChanged(const QDBusObjectPath & _t1, const QVariantMap & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
QT_WARNING_POP
