// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "plugin_screensaver_inhibit_debug.h"


Q_LOGGING_CATEGORY(KDECONNECT_PLUGIN_SCREENSAVER_INHIBIT, "kdeconnect.plugin.screensaver_inhibit", QtWarningMsg)

