/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was mprisremoteinterface.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef MPRISREMOTEINTERFACE_H
#define MPRISREMOTEINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.device.mprisremote
 */
class OrgKdeKdeconnectDeviceMprisremoteInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kdeconnect.device.mprisremote"; }

public:
    OrgKdeKdeconnectDeviceMprisremoteInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDeviceMprisremoteInterface();

    Q_PROPERTY(QString album READ album)
    inline QString album() const
    { return qvariant_cast< QString >(property("album")); }

    Q_PROPERTY(QString artist READ artist)
    inline QString artist() const
    { return qvariant_cast< QString >(property("artist")); }

    Q_PROPERTY(bool canSeek READ canSeek)
    inline bool canSeek() const
    { return qvariant_cast< bool >(property("canSeek")); }

    Q_PROPERTY(bool isPlaying READ isPlaying)
    inline bool isPlaying() const
    { return qvariant_cast< bool >(property("isPlaying")); }

    Q_PROPERTY(int length READ length)
    inline int length() const
    { return qvariant_cast< int >(property("length")); }

    Q_PROPERTY(QString localAlbumArtUrl READ localAlbumArtUrl)
    inline QString localAlbumArtUrl() const
    { return qvariant_cast< QString >(property("localAlbumArtUrl")); }

    Q_PROPERTY(QString player READ player WRITE setPlayer)
    inline QString player() const
    { return qvariant_cast< QString >(property("player")); }
    inline void setPlayer(const QString &value)
    { setProperty("player", QVariant::fromValue(value)); }

    Q_PROPERTY(QStringList playerList READ playerList)
    inline QStringList playerList() const
    { return qvariant_cast< QStringList >(property("playerList")); }

    Q_PROPERTY(int position READ position WRITE setPosition)
    inline int position() const
    { return qvariant_cast< int >(property("position")); }
    inline void setPosition(int value)
    { setProperty("position", QVariant::fromValue(value)); }

    Q_PROPERTY(QString title READ title)
    inline QString title() const
    { return qvariant_cast< QString >(property("title")); }

    Q_PROPERTY(int volume READ volume WRITE setVolume)
    inline int volume() const
    { return qvariant_cast< int >(property("volume")); }
    inline void setVolume(int value)
    { setProperty("volume", QVariant::fromValue(value)); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> requestPlayerList()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("requestPlayerList"), argumentList);
    }

    inline QDBusPendingReply<> seek(int offset)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(offset);
        return asyncCallWithArgumentList(QStringLiteral("seek"), argumentList);
    }

    inline QDBusPendingReply<> sendAction(const QString &action)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(action);
        return asyncCallWithArgumentList(QStringLiteral("sendAction"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void propertiesChanged();
};

#endif
// clazy:skip
