/****************************************************************************
** Meta object code from reading C++ file 'smshelper.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kdeconnect-kde-26.04.3/smsapp/smshelper.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smshelper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9SmsHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto SmsHelper::qt_create_metaobjectdata<qt_meta_tag_ZN9SmsHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SmsHelper",
        "isPhoneNumberMatch",
        "",
        "phone1",
        "phone2",
        "isPhoneNumberMatchCanonicalized",
        "canonicalPhone1",
        "canonicalPhone2",
        "isShortCode",
        "canonicalNumber",
        "CountryCode",
        "country",
        "canonicalizePhoneNumber",
        "phoneNumber",
        "lookupPersonByAddress",
        "QSharedPointer<KPeople::PersonData>",
        "address",
        "combineIcons",
        "QIcon",
        "QList<QPixmap>",
        "icons",
        "getTitleForAddresses",
        "QList<ConversationAddress>",
        "addresses",
        "getIconForAddresses",
        "copyToClipboard",
        "text",
        "getCharCount",
        "SmsCharCount",
        "message",
        "isAddressValid",
        "totalMessageSize",
        "QList<QUrl>",
        "urls",
        "getThumbnailForAttachment",
        "Attachment",
        "attachment"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'isPhoneNumberMatch'
        QtMocHelpers::MethodData<bool(const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Method 'isPhoneNumberMatchCanonicalized'
        QtMocHelpers::MethodData<bool(const QString &, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 6 }, { QMetaType::QString, 7 },
        }}),
        // Method 'isShortCode'
        QtMocHelpers::MethodData<bool(const QString &, const CountryCode &)>(8, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Method 'canonicalizePhoneNumber'
        QtMocHelpers::MethodData<QString(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 13 },
        }}),
        // Method 'lookupPersonByAddress'
        QtMocHelpers::MethodData<QSharedPointer<KPeople::PersonData>(const QString &)>(14, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QString, 16 },
        }}),
        // Method 'combineIcons'
        QtMocHelpers::MethodData<QIcon(const QList<QPixmap> &)>(17, 2, QMC::AccessPublic, 0x80000000 | 18, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Method 'getTitleForAddresses'
        QtMocHelpers::MethodData<QString(const QList<ConversationAddress> &)>(21, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Method 'getIconForAddresses'
        QtMocHelpers::MethodData<QIcon(const QList<ConversationAddress> &)>(24, 2, QMC::AccessPublic, 0x80000000 | 18, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Method 'copyToClipboard'
        QtMocHelpers::MethodData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 },
        }}),
        // Method 'getCharCount'
        QtMocHelpers::MethodData<SmsCharCount(const QString &)>(27, 2, QMC::AccessPublic, 0x80000000 | 28, {{
            { QMetaType::QString, 29 },
        }}),
        // Method 'isAddressValid'
        QtMocHelpers::MethodData<bool(const QString &)>(30, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 16 },
        }}),
        // Method 'totalMessageSize'
        QtMocHelpers::MethodData<quint64(const QList<QUrl> &, const QString &)>(31, 2, QMC::AccessPublic, QMetaType::ULongLong, {{
            { 0x80000000 | 32, 33 }, { QMetaType::QString, 26 },
        }}),
        // Method 'getThumbnailForAttachment'
        QtMocHelpers::MethodData<QIcon(const Attachment &)>(34, 2, QMC::AccessPublic, 0x80000000 | 18, {{
            { 0x80000000 | 35, 36 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SmsHelper, qt_meta_tag_ZN9SmsHelperE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SmsHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9SmsHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9SmsHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9SmsHelperE_t>.metaTypes,
    nullptr
} };

void SmsHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SmsHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { bool _r = _t->isPhoneNumberMatch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->isPhoneNumberMatchCanonicalized((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->isShortCode((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<enum CountryCode>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { QString _r = _t->canonicalizePhoneNumber((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 4: { QSharedPointer<KPeople::PersonData> _r = _t->lookupPersonByAddress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QSharedPointer<KPeople::PersonData>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QIcon _r = _t->combineIcons((*reinterpret_cast<std::add_pointer_t<QList<QPixmap>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QIcon*>(_a[0]) = std::move(_r); }  break;
        case 6: { QString _r = _t->getTitleForAddresses((*reinterpret_cast<std::add_pointer_t<QList<ConversationAddress>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 7: { QIcon _r = _t->getIconForAddresses((*reinterpret_cast<std::add_pointer_t<QList<ConversationAddress>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QIcon*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->copyToClipboard((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: { SmsCharCount _r = _t->getCharCount((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<SmsCharCount*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->isAddressValid((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { quint64 _r = _t->totalMessageSize((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<quint64*>(_a[0]) = std::move(_r); }  break;
        case 12: { QIcon _r = _t->getThumbnailForAttachment((*reinterpret_cast<std::add_pointer_t<Attachment>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QIcon*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<ConversationAddress> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<ConversationAddress> >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Attachment >(); break;
            }
            break;
        }
    }
}

const QMetaObject *SmsHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SmsHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9SmsHelperE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SmsHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
