/****************************************************************************
** Meta object code from reading C++ file 'avahientrygroup.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "avahientrygroup.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'avahientrygroup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN38OrgFreedesktopAvahiEntryGroupInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopAvahiEntryGroupInterface::qt_create_metaobjectdata<qt_meta_tag_ZN38OrgFreedesktopAvahiEntryGroupInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopAvahiEntryGroupInterface",
        "StateChanged",
        "",
        "state",
        "error",
        "AddAddress",
        "QDBusPendingReply<>",
        "interface",
        "protocol",
        "flags",
        "name",
        "address",
        "AddRecord",
        "clazz",
        "type",
        "ttl",
        "rdata",
        "AddService",
        "domain",
        "host",
        "port",
        "QByteArrayList",
        "txt",
        "AddServiceSubtype",
        "subtype",
        "Commit",
        "Free",
        "GetState",
        "QDBusPendingReply<int>",
        "IsEmpty",
        "QDBusPendingReply<bool>",
        "Reset",
        "UpdateServiceTxt"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'StateChanged'
        QtMocHelpers::SignalData<void(int, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 4 },
        }}),
        // Slot 'AddAddress'
        QtMocHelpers::SlotData<QDBusPendingReply<>(int, int, uint, const QString &, const QString &)>(5, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::UInt, 9 }, { QMetaType::QString, 10 },
            { QMetaType::QString, 11 },
        }}),
        // Slot 'AddRecord'
        QtMocHelpers::SlotData<QDBusPendingReply<>(int, int, uint, const QString &, ushort, ushort, uint, const QByteArray &)>(12, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::UInt, 9 }, { QMetaType::QString, 10 },
            { QMetaType::UShort, 13 }, { QMetaType::UShort, 14 }, { QMetaType::UInt, 15 }, { QMetaType::QByteArray, 16 },
        }}),
        // Slot 'AddService'
        QtMocHelpers::SlotData<QDBusPendingReply<>(int, int, uint, const QString &, const QString &, const QString &, const QString &, ushort, const QByteArrayList &)>(17, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::UInt, 9 }, { QMetaType::QString, 10 },
            { QMetaType::QString, 14 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::UShort, 20 },
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'AddServiceSubtype'
        QtMocHelpers::SlotData<QDBusPendingReply<>(int, int, uint, const QString &, const QString &, const QString &, const QString &)>(23, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::UInt, 9 }, { QMetaType::QString, 10 },
            { QMetaType::QString, 14 }, { QMetaType::QString, 18 }, { QMetaType::QString, 24 },
        }}),
        // Slot 'Commit'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(25, 2, QMC::AccessPublic, 0x80000000 | 6),
        // Slot 'Free'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(26, 2, QMC::AccessPublic, 0x80000000 | 6),
        // Slot 'GetState'
        QtMocHelpers::SlotData<QDBusPendingReply<int>()>(27, 2, QMC::AccessPublic, 0x80000000 | 28),
        // Slot 'IsEmpty'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>()>(29, 2, QMC::AccessPublic, 0x80000000 | 30),
        // Slot 'Reset'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(31, 2, QMC::AccessPublic, 0x80000000 | 6),
        // Slot 'UpdateServiceTxt'
        QtMocHelpers::SlotData<QDBusPendingReply<>(int, int, uint, const QString &, const QString &, const QString &, const QByteArrayList &)>(32, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::UInt, 9 }, { QMetaType::QString, 10 },
            { QMetaType::QString, 14 }, { QMetaType::QString, 18 }, { 0x80000000 | 21, 22 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopAvahiEntryGroupInterface, qt_meta_tag_ZN38OrgFreedesktopAvahiEntryGroupInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopAvahiEntryGroupInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN38OrgFreedesktopAvahiEntryGroupInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN38OrgFreedesktopAvahiEntryGroupInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN38OrgFreedesktopAvahiEntryGroupInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopAvahiEntryGroupInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopAvahiEntryGroupInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->StateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: { QDBusPendingReply<> _r = _t->AddAddress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<> _r = _t->AddRecord((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<ushort>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<ushort>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[8])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<> _r = _t->AddService((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<ushort>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QByteArrayList>>(_a[9])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<> _r = _t->AddServiceSubtype((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->Commit();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->Free();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<int> _r = _t->GetState();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<bool> _r = _t->IsEmpty();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<> _r = _t->Reset();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<> _r = _t->UpdateServiceTxt((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QByteArrayList>>(_a[7])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopAvahiEntryGroupInterface::*)(int , const QString & )>(_a, &OrgFreedesktopAvahiEntryGroupInterface::StateChanged, 0))
            return;
    }
}

const QMetaObject *OrgFreedesktopAvahiEntryGroupInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopAvahiEntryGroupInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN38OrgFreedesktopAvahiEntryGroupInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopAvahiEntryGroupInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void OrgFreedesktopAvahiEntryGroupInterface::StateChanged(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
