/****************************************************************************
** Meta object code from reading C++ file 'conversationsinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "conversationsinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'conversationsinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN44OrgKdeKdeconnectDeviceConversationsInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgKdeKdeconnectDeviceConversationsInterface::qt_create_metaobjectdata<qt_meta_tag_ZN44OrgKdeKdeconnectDeviceConversationsInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgKdeKdeconnectDeviceConversationsInterface",
        "attachmentReceived",
        "",
        "filePath",
        "fileName",
        "conversationCreated",
        "QDBusVariant",
        "msg",
        "conversationLoaded",
        "conversationID",
        "messageCount",
        "conversationRemoved",
        "conversationUpdated",
        "activeConversations",
        "QDBusPendingReply<QVariantList>",
        "replyToConversation",
        "QDBusPendingReply<>",
        "message",
        "QVariantList",
        "attachmentUrls",
        "requestAllConversationThreads",
        "requestAttachmentFile",
        "partID",
        "uniqueIdentifier",
        "requestConversation",
        "start",
        "end",
        "sendWithoutConversation",
        "addressList"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'attachmentReceived'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'conversationCreated'
        QtMocHelpers::SignalData<void(const QDBusVariant &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'conversationLoaded'
        QtMocHelpers::SignalData<void(qlonglong, qulonglong)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 9 }, { QMetaType::ULongLong, 10 },
        }}),
        // Signal 'conversationRemoved'
        QtMocHelpers::SignalData<void(qlonglong)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 9 },
        }}),
        // Signal 'conversationUpdated'
        QtMocHelpers::SignalData<void(const QDBusVariant &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'activeConversations'
        QtMocHelpers::SlotData<QDBusPendingReply<QVariantList>()>(13, 2, QMC::AccessPublic, 0x80000000 | 14),
        // Slot 'replyToConversation'
        QtMocHelpers::SlotData<QDBusPendingReply<>(qlonglong, const QString &, const QVariantList &)>(15, 2, QMC::AccessPublic, 0x80000000 | 16, {{
            { QMetaType::LongLong, 9 }, { QMetaType::QString, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'requestAllConversationThreads'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(20, 2, QMC::AccessPublic, 0x80000000 | 16),
        // Slot 'requestAttachmentFile'
        QtMocHelpers::SlotData<QDBusPendingReply<>(qlonglong, const QString &)>(21, 2, QMC::AccessPublic, 0x80000000 | 16, {{
            { QMetaType::LongLong, 22 }, { QMetaType::QString, 23 },
        }}),
        // Slot 'requestConversation'
        QtMocHelpers::SlotData<QDBusPendingReply<>(qlonglong, int, int)>(24, 2, QMC::AccessPublic, 0x80000000 | 16, {{
            { QMetaType::LongLong, 9 }, { QMetaType::Int, 25 }, { QMetaType::Int, 26 },
        }}),
        // Slot 'sendWithoutConversation'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QVariantList &, const QString &, const QVariantList &)>(27, 2, QMC::AccessPublic, 0x80000000 | 16, {{
            { 0x80000000 | 18, 28 }, { QMetaType::QString, 17 }, { 0x80000000 | 18, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgKdeKdeconnectDeviceConversationsInterface, qt_meta_tag_ZN44OrgKdeKdeconnectDeviceConversationsInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgKdeKdeconnectDeviceConversationsInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN44OrgKdeKdeconnectDeviceConversationsInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN44OrgKdeKdeconnectDeviceConversationsInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN44OrgKdeKdeconnectDeviceConversationsInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgKdeKdeconnectDeviceConversationsInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgKdeKdeconnectDeviceConversationsInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->attachmentReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->conversationCreated((*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[1]))); break;
        case 2: _t->conversationLoaded((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[2]))); break;
        case 3: _t->conversationRemoved((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1]))); break;
        case 4: _t->conversationUpdated((*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[1]))); break;
        case 5: { QDBusPendingReply<QVariantList> _r = _t->activeConversations();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QVariantList>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->replyToConversation((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->requestAllConversationThreads();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<> _r = _t->requestAttachmentFile((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<> _r = _t->requestConversation((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<> _r = _t->sendWithoutConversation((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceConversationsInterface::*)(const QString & , const QString & )>(_a, &OrgKdeKdeconnectDeviceConversationsInterface::attachmentReceived, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceConversationsInterface::*)(const QDBusVariant & )>(_a, &OrgKdeKdeconnectDeviceConversationsInterface::conversationCreated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceConversationsInterface::*)(qlonglong , qulonglong )>(_a, &OrgKdeKdeconnectDeviceConversationsInterface::conversationLoaded, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceConversationsInterface::*)(qlonglong )>(_a, &OrgKdeKdeconnectDeviceConversationsInterface::conversationRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceConversationsInterface::*)(const QDBusVariant & )>(_a, &OrgKdeKdeconnectDeviceConversationsInterface::conversationUpdated, 4))
            return;
    }
}

const QMetaObject *OrgKdeKdeconnectDeviceConversationsInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgKdeKdeconnectDeviceConversationsInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN44OrgKdeKdeconnectDeviceConversationsInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgKdeKdeconnectDeviceConversationsInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void OrgKdeKdeconnectDeviceConversationsInterface::attachmentReceived(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OrgKdeKdeconnectDeviceConversationsInterface::conversationCreated(const QDBusVariant & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OrgKdeKdeconnectDeviceConversationsInterface::conversationLoaded(qlonglong _t1, qulonglong _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OrgKdeKdeconnectDeviceConversationsInterface::conversationRemoved(qlonglong _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OrgKdeKdeconnectDeviceConversationsInterface::conversationUpdated(const QDBusVariant & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
