// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "sms_conversations_list_debug.h"


Q_LOGGING_CATEGORY(KDECONNECT_SMS_CONVERSATIONS_LIST_MODEL, "kdeconnect.sms.conversations_list", QtWarningMsg)

