/****************************************************************************
** Meta object code from reading C++ file 'devicesftpinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "devicesftpinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'devicesftpinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN35OrgKdeKdeconnectDeviceSftpInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgKdeKdeconnectDeviceSftpInterface::qt_create_metaobjectdata<qt_meta_tag_ZN35OrgKdeKdeconnectDeviceSftpInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgKdeKdeconnectDeviceSftpInterface",
        "mounted",
        "",
        "unmounted",
        "getDirectories",
        "QDBusPendingReply<QVariantMap>",
        "getMountError",
        "QDBusPendingReply<QString>",
        "isMounted",
        "QDBusPendingReply<bool>",
        "mount",
        "QDBusPendingReply<>",
        "mountAndWait",
        "mountPoint",
        "startBrowsing",
        "unmount"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'mounted'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'unmounted'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getDirectories'
        QtMocHelpers::SlotData<QDBusPendingReply<QVariantMap>()>(4, 2, QMC::AccessPublic, 0x80000000 | 5),
        // Slot 'getMountError'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(6, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'isMounted'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>()>(8, 2, QMC::AccessPublic, 0x80000000 | 9),
        // Slot 'mount'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(10, 2, QMC::AccessPublic, 0x80000000 | 11),
        // Slot 'mountAndWait'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>()>(12, 2, QMC::AccessPublic, 0x80000000 | 9),
        // Slot 'mountPoint'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(13, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'startBrowsing'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>()>(14, 2, QMC::AccessPublic, 0x80000000 | 9),
        // Slot 'unmount'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(15, 2, QMC::AccessPublic, 0x80000000 | 11),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgKdeKdeconnectDeviceSftpInterface, qt_meta_tag_ZN35OrgKdeKdeconnectDeviceSftpInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgKdeKdeconnectDeviceSftpInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35OrgKdeKdeconnectDeviceSftpInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35OrgKdeKdeconnectDeviceSftpInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN35OrgKdeKdeconnectDeviceSftpInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgKdeKdeconnectDeviceSftpInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgKdeKdeconnectDeviceSftpInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->mounted(); break;
        case 1: _t->unmounted(); break;
        case 2: { QDBusPendingReply<QVariantMap> _r = _t->getDirectories();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<QString> _r = _t->getMountError();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<bool> _r = _t->isMounted();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->mount();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<bool> _r = _t->mountAndWait();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<QString> _r = _t->mountPoint();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<bool> _r = _t->startBrowsing();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<> _r = _t->unmount();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceSftpInterface::*)()>(_a, &OrgKdeKdeconnectDeviceSftpInterface::mounted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceSftpInterface::*)()>(_a, &OrgKdeKdeconnectDeviceSftpInterface::unmounted, 1))
            return;
    }
}

const QMetaObject *OrgKdeKdeconnectDeviceSftpInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgKdeKdeconnectDeviceSftpInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35OrgKdeKdeconnectDeviceSftpInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgKdeKdeconnectDeviceSftpInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void OrgKdeKdeconnectDeviceSftpInterface::mounted()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OrgKdeKdeconnectDeviceSftpInterface::unmounted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
