/****************************************************************************
** Meta object code from reading C++ file 'daemon.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kdeconnect-kde-26.04.3/core/daemon.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'daemon.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6DaemonE_t {};
} // unnamed namespace

template <> constexpr inline auto Daemon::qt_create_metaobjectdata<qt_meta_tag_ZN6DaemonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Daemon",
        "D-Bus Interface",
        "org.kde.kdeconnect.daemon",
        "deviceAdded",
        "",
        "id",
        "deviceRemoved",
        "deviceVisibilityChanged",
        "isVisible",
        "deviceListChanged",
        "announcedNameChanged",
        "announcedName",
        "pairingRequestsChanged",
        "linkProvidersChanged",
        "linkProviders",
        "customDevicesChanged",
        "customDevices",
        "forceOnNetworkChange",
        "setAnnouncedName",
        "name",
        "devices",
        "onlyReachable",
        "onlyPaired",
        "deviceNames",
        "QMap<QString,QString>",
        "deviceIdByName",
        "setLinkProviderState",
        "linkProvider",
        "enabled",
        "sendSimpleNotification",
        "eventId",
        "title",
        "text",
        "iconName",
        "onNewDeviceLink",
        "DeviceLink*",
        "dl",
        "onDeviceStatusChanged",
        "selfId",
        "pairingRequests"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'deviceAdded'
        QtMocHelpers::SignalData<void(const QString &)>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'deviceRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(6, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'deviceVisibilityChanged'
        QtMocHelpers::SignalData<void(const QString &, bool)>(7, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { QMetaType::Bool, 8 },
        }}),
        // Signal 'deviceListChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Signal 'announcedNameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(10, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'pairingRequestsChanged'
        QtMocHelpers::SignalData<void()>(12, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Signal 'linkProvidersChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(13, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QStringList, 14 },
        }}),
        // Signal 'customDevicesChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(15, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QStringList, 16 },
        }}),
        // Slot 'forceOnNetworkChange'
        QtMocHelpers::SlotData<void()>(17, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'announcedName'
        QtMocHelpers::SlotData<QString()>(11, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'setAnnouncedName'
        QtMocHelpers::SlotData<void(const QString &)>(18, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'devices'
        QtMocHelpers::SlotData<QStringList(bool, bool) const>(20, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QStringList, {{
            { QMetaType::Bool, 21 }, { QMetaType::Bool, 22 },
        }}),
        // Slot 'devices'
        QtMocHelpers::SlotData<QStringList(bool) const>(20, 4, QMC::AccessPublic | QMC::MethodCloned | QMC::MethodScriptable, QMetaType::QStringList, {{
            { QMetaType::Bool, 21 },
        }}),
        // Slot 'devices'
        QtMocHelpers::SlotData<QStringList() const>(20, 4, QMC::AccessPublic | QMC::MethodCloned | QMC::MethodScriptable, QMetaType::QStringList),
        // Slot 'deviceNames'
        QtMocHelpers::SlotData<QMap<QString,QString>(bool, bool) const>(23, 4, QMC::AccessPublic | QMC::MethodScriptable, 0x80000000 | 24, {{
            { QMetaType::Bool, 21 }, { QMetaType::Bool, 22 },
        }}),
        // Slot 'deviceNames'
        QtMocHelpers::SlotData<QMap<QString,QString>(bool) const>(23, 4, QMC::AccessPublic | QMC::MethodCloned | QMC::MethodScriptable, 0x80000000 | 24, {{
            { QMetaType::Bool, 21 },
        }}),
        // Slot 'deviceNames'
        QtMocHelpers::SlotData<QMap<QString,QString>() const>(23, 4, QMC::AccessPublic | QMC::MethodCloned | QMC::MethodScriptable, 0x80000000 | 24),
        // Slot 'deviceIdByName'
        QtMocHelpers::SlotData<QString(const QString &) const>(25, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'linkProviders'
        QtMocHelpers::SlotData<QStringList() const>(14, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QStringList),
        // Slot 'setLinkProviderState'
        QtMocHelpers::SlotData<void(const QString &, bool)>(26, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 27 }, { QMetaType::Bool, 28 },
        }}),
        // Slot 'sendSimpleNotification'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, const QString &)>(29, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 30 }, { QMetaType::QString, 31 }, { QMetaType::QString, 32 }, { QMetaType::QString, 33 },
        }}),
        // Slot 'onNewDeviceLink'
        QtMocHelpers::SlotData<void(DeviceLink *)>(34, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 35, 36 },
        }}),
        // Slot 'onDeviceStatusChanged'
        QtMocHelpers::SlotData<void()>(37, 4, QMC::AccessPrivate, QMetaType::Void),
        // Method 'selfId'
        QtMocHelpers::MethodData<QString() const>(38, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'pairingRequests'
        QtMocHelpers::PropertyData<QStringList>(39, QMetaType::QStringList, QMC::DefaultPropertyFlags, 5),
        // property 'customDevices'
        QtMocHelpers::PropertyData<QStringList>(16, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<Daemon, qt_meta_tag_ZN6DaemonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Daemon::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6DaemonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6DaemonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6DaemonE_t>.metaTypes,
    nullptr
} };

void Daemon::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Daemon *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->deviceAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->deviceRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->deviceVisibilityChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->deviceListChanged(); break;
        case 4: _t->announcedNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->pairingRequestsChanged(); break;
        case 6: _t->linkProvidersChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 7: _t->customDevicesChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 8: _t->forceOnNetworkChange(); break;
        case 9: { QString _r = _t->announcedName();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->setAnnouncedName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: { QStringList _r = _t->devices((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 12: { QStringList _r = _t->devices((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 13: { QStringList _r = _t->devices();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 14: { QMap<QString,QString> _r = _t->deviceNames((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QMap<QString,QString>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QMap<QString,QString> _r = _t->deviceNames((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QMap<QString,QString>*>(_a[0]) = std::move(_r); }  break;
        case 16: { QMap<QString,QString> _r = _t->deviceNames();
            if (_a[0]) *reinterpret_cast<QMap<QString,QString>*>(_a[0]) = std::move(_r); }  break;
        case 17: { QString _r = _t->deviceIdByName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 18: { QStringList _r = _t->linkProviders();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->setLinkProviderState((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 20: _t->sendSimpleNotification((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 21: _t->onNewDeviceLink((*reinterpret_cast<std::add_pointer_t<DeviceLink*>>(_a[1]))); break;
        case 22: _t->onDeviceStatusChanged(); break;
        case 23: { QString _r = _t->selfId();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeviceLink* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Daemon::*)(const QString & )>(_a, &Daemon::deviceAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Daemon::*)(const QString & )>(_a, &Daemon::deviceRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Daemon::*)(const QString & , bool )>(_a, &Daemon::deviceVisibilityChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Daemon::*)()>(_a, &Daemon::deviceListChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Daemon::*)(const QString & )>(_a, &Daemon::announcedNameChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Daemon::*)()>(_a, &Daemon::pairingRequestsChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Daemon::*)(const QStringList & )>(_a, &Daemon::linkProvidersChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Daemon::*)(const QStringList & )>(_a, &Daemon::customDevicesChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QStringList*>(_v) = _t->pairingRequests(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->customDevices(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setCustomDevices(*reinterpret_cast<QStringList*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Daemon::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Daemon::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6DaemonE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Daemon::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void Daemon::deviceAdded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Daemon::deviceRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Daemon::deviceVisibilityChanged(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void Daemon::deviceListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Daemon::announcedNameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Daemon::pairingRequestsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Daemon::linkProvidersChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Daemon::customDevicesChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
