// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "sms_conversation_debug.h"


Q_LOGGING_CATEGORY(KDECONNECT_SMS_CONVERSATION_MODEL, "kdeconnect.sms.conversation", QtWarningMsg)

