/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was smsinterface.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef SMSINTERFACE_H
#define SMSINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.device.sms
 */
class OrgKdeKdeconnectDeviceSmsInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kdeconnect.device.sms"; }

public:
    OrgKdeKdeconnectDeviceSmsInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDeviceSmsInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> getAttachment(qlonglong partID, const QString &uniqueIdentifier)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(partID) << QVariant::fromValue(uniqueIdentifier);
        return asyncCallWithArgumentList(QStringLiteral("getAttachment"), argumentList);
    }

    inline QDBusPendingReply<> launchApp()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("launchApp"), argumentList);
    }

    inline QDBusPendingReply<> requestAllConversations()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("requestAllConversations"), argumentList);
    }

    inline QDBusPendingReply<> requestAttachment(qlonglong partID, const QString &uniqueIdentifier)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(partID) << QVariant::fromValue(uniqueIdentifier);
        return asyncCallWithArgumentList(QStringLiteral("requestAttachment"), argumentList);
    }

    inline QDBusPendingReply<> requestConversation(qlonglong conversationID)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(conversationID);
        return asyncCallWithArgumentList(QStringLiteral("requestConversation"), argumentList);
    }

    inline QDBusPendingReply<> requestConversation(qlonglong conversationID, qlonglong rangeStartTimestamp)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(conversationID) << QVariant::fromValue(rangeStartTimestamp);
        return asyncCallWithArgumentList(QStringLiteral("requestConversation"), argumentList);
    }

    inline QDBusPendingReply<> requestConversation(qlonglong conversationID, qlonglong rangeStartTimestamp, qlonglong numberToRequest)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(conversationID) << QVariant::fromValue(rangeStartTimestamp) << QVariant::fromValue(numberToRequest);
        return asyncCallWithArgumentList(QStringLiteral("requestConversation"), argumentList);
    }

    inline QDBusPendingReply<> sendSms(const QVariantList &addresses, const QString &textMessage, const QVariantList &attachmentUrls)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(addresses) << QVariant::fromValue(textMessage) << QVariant::fromValue(attachmentUrls);
        return asyncCallWithArgumentList(QStringLiteral("sendSms"), argumentList);
    }

    inline QDBusPendingReply<> sendSms(const QVariantList &addresses, const QString &textMessage, const QVariantList &attachmentUrls, qlonglong subID)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(addresses) << QVariant::fromValue(textMessage) << QVariant::fromValue(attachmentUrls) << QVariant::fromValue(subID);
        return asyncCallWithArgumentList(QStringLiteral("sendSms"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

#endif
// clazy:skip
