/****************************************************************************
** Meta object code from reading C++ file 'conversationsdbusinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kdeconnect-kde-26.04.3/plugins/sms/conversationsdbusinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'conversationsdbusinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26ConversationsDbusInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto ConversationsDbusInterface::qt_create_metaobjectdata<qt_meta_tag_ZN26ConversationsDbusInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ConversationsDbusInterface",
        "D-Bus Interface",
        "org.kde.kdeconnect.device.conversations",
        "veryHackyWorkaround",
        "",
        "QVariant",
        "conversationCreated",
        "QDBusVariant",
        "msg",
        "conversationRemoved",
        "conversationID",
        "conversationUpdated",
        "conversationLoaded",
        "messageCount",
        "attachmentReceived",
        "filePath",
        "fileName",
        "activeConversations",
        "QVariantList",
        "requestConversation",
        "start",
        "end",
        "replyToConversation",
        "message",
        "attachmentUrls",
        "sendWithoutConversation",
        "addressList",
        "requestAllConversationThreads",
        "requestAttachmentFile",
        "partID",
        "uniqueIdentifier"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'veryHackyWorkaround'
        QtMocHelpers::SignalData<void(const QVariant &)>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { 0x80000000 | 5, 4 },
        }}),
        // Signal 'conversationCreated'
        QtMocHelpers::SignalData<void(const QDBusVariant &)>(6, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'conversationRemoved'
        QtMocHelpers::SignalData<void(const qint64 &)>(9, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::LongLong, 10 },
        }}),
        // Signal 'conversationUpdated'
        QtMocHelpers::SignalData<void(const QDBusVariant &)>(11, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'conversationLoaded'
        QtMocHelpers::SignalData<void(qint64, quint64)>(12, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::LongLong, 10 }, { QMetaType::ULongLong, 13 },
        }}),
        // Signal 'attachmentReceived'
        QtMocHelpers::SignalData<void(QString, QString)>(14, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::QString, 16 },
        }}),
        // Slot 'activeConversations'
        QtMocHelpers::SlotData<QVariantList()>(17, 4, QMC::AccessPublic, 0x80000000 | 18),
        // Slot 'requestConversation'
        QtMocHelpers::SlotData<void(const qint64 &, int, int)>(19, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 10 }, { QMetaType::Int, 20 }, { QMetaType::Int, 21 },
        }}),
        // Slot 'replyToConversation'
        QtMocHelpers::SlotData<void(const qint64 &, const QString &, const QVariantList &)>(22, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 10 }, { QMetaType::QString, 23 }, { 0x80000000 | 18, 24 },
        }}),
        // Slot 'sendWithoutConversation'
        QtMocHelpers::SlotData<void(const QVariantList &, const QString &, const QVariantList &)>(25, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 26 }, { QMetaType::QString, 23 }, { 0x80000000 | 18, 24 },
        }}),
        // Slot 'requestAllConversationThreads'
        QtMocHelpers::SlotData<void()>(27, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'requestAttachmentFile'
        QtMocHelpers::SlotData<void(const qint64 &, const QString &)>(28, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 29 }, { QMetaType::QString, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<ConversationsDbusInterface, qt_meta_tag_ZN26ConversationsDbusInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject ConversationsDbusInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26ConversationsDbusInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26ConversationsDbusInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26ConversationsDbusInterfaceE_t>.metaTypes,
    nullptr
} };

void ConversationsDbusInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConversationsDbusInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->veryHackyWorkaround((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        case 1: _t->conversationCreated((*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[1]))); break;
        case 2: _t->conversationRemoved((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 3: _t->conversationUpdated((*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[1]))); break;
        case 4: _t->conversationLoaded((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint64>>(_a[2]))); break;
        case 5: _t->attachmentReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: { QVariantList _r = _t->activeConversations();
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->requestConversation((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 8: _t->replyToConversation((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[3]))); break;
        case 9: _t->sendWithoutConversation((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[3]))); break;
        case 10: _t->requestAllConversationThreads(); break;
        case 11: _t->requestAttachmentFile((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusVariant >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusVariant >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ConversationsDbusInterface::*)(const QVariant & )>(_a, &ConversationsDbusInterface::veryHackyWorkaround, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConversationsDbusInterface::*)(const QDBusVariant & )>(_a, &ConversationsDbusInterface::conversationCreated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConversationsDbusInterface::*)(const qint64 & )>(_a, &ConversationsDbusInterface::conversationRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConversationsDbusInterface::*)(const QDBusVariant & )>(_a, &ConversationsDbusInterface::conversationUpdated, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConversationsDbusInterface::*)(qint64 , quint64 )>(_a, &ConversationsDbusInterface::conversationLoaded, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ConversationsDbusInterface::*)(QString , QString )>(_a, &ConversationsDbusInterface::attachmentReceived, 5))
            return;
    }
}

const QMetaObject *ConversationsDbusInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ConversationsDbusInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26ConversationsDbusInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int ConversationsDbusInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void ConversationsDbusInterface::veryHackyWorkaround(const QVariant & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ConversationsDbusInterface::conversationCreated(const QDBusVariant & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ConversationsDbusInterface::conversationRemoved(const qint64 & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ConversationsDbusInterface::conversationUpdated(const QDBusVariant & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void ConversationsDbusInterface::conversationLoaded(qint64 _t1, quint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void ConversationsDbusInterface::attachmentReceived(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}
QT_WARNING_POP
