/****************************************************************************
** Meta object code from reading C++ file 'device.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kdeconnect-kde-26.04.3/core/device.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'device.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6DeviceE_t {};
} // unnamed namespace

template <> constexpr inline auto Device::qt_create_metaobjectdata<qt_meta_tag_ZN6DeviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Device",
        "D-Bus Interface",
        "org.kde.kdeconnect.device",
        "pluginsChanged",
        "",
        "linksChanged",
        "reachableChanged",
        "reachable",
        "pairStateChanged",
        "pairState",
        "pairingFailed",
        "error",
        "nameChanged",
        "name",
        "typeChanged",
        "type",
        "statusIconNameChanged",
        "sendPacket",
        "NetworkPacket&",
        "np",
        "requestPairing",
        "unpair",
        "reloadPlugins",
        "acceptPairing",
        "cancelPairing",
        "pluginIconName",
        "pluginName",
        "privateReceivedPacket",
        "NetworkPacket",
        "linkDestroyed",
        "o",
        "pairingHandler_incomingPairRequest",
        "pairingHandler_pairingFailed",
        "errorMessage",
        "pairingHandler_pairingSuccessful",
        "pairingHandler_unpaired",
        "verificationKey",
        "encryptionInfo",
        "pairStateAsInt",
        "isPaired",
        "isPairRequested",
        "isPairRequestedByPeer",
        "loadedPlugins",
        "hasPlugin",
        "pluginsConfigFile",
        "setPluginEnabled",
        "enabled",
        "isPluginEnabled",
        "iconName",
        "statusIconName",
        "isReachable",
        "reachableAddresses",
        "activeProviderNames",
        "supportedPlugins"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pluginsChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Signal 'linksChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Signal 'reachableChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'pairStateChanged'
        QtMocHelpers::SignalData<void(int)>(8, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Signal 'pairingFailed'
        QtMocHelpers::SignalData<void(const QString &)>(10, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'typeChanged'
        QtMocHelpers::SignalData<void(const QString &)>(14, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Signal 'statusIconNameChanged'
        QtMocHelpers::SignalData<void()>(16, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'sendPacket'
        QtMocHelpers::SlotData<bool(NetworkPacket &)>(17, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'requestPairing'
        QtMocHelpers::SlotData<void()>(20, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'unpair'
        QtMocHelpers::SlotData<void()>(21, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'reloadPlugins'
        QtMocHelpers::SlotData<void()>(22, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'acceptPairing'
        QtMocHelpers::SlotData<void()>(23, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'cancelPairing'
        QtMocHelpers::SlotData<void()>(24, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'pluginIconName'
        QtMocHelpers::SlotData<QString(const QString &)>(25, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString, {{
            { QMetaType::QString, 26 },
        }}),
        // Slot 'privateReceivedPacket'
        QtMocHelpers::SlotData<void(const NetworkPacket &)>(27, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 19 },
        }}),
        // Slot 'linkDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(29, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 30 },
        }}),
        // Slot 'pairingHandler_incomingPairRequest'
        QtMocHelpers::SlotData<void()>(31, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'pairingHandler_pairingFailed'
        QtMocHelpers::SlotData<void(const QString &)>(32, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Slot 'pairingHandler_pairingSuccessful'
        QtMocHelpers::SlotData<void()>(34, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'pairingHandler_unpaired'
        QtMocHelpers::SlotData<void()>(35, 4, QMC::AccessPrivate, QMetaType::Void),
        // Method 'verificationKey'
        QtMocHelpers::MethodData<QString() const>(36, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Method 'encryptionInfo'
        QtMocHelpers::MethodData<QString() const>(37, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Method 'pairStateAsInt'
        QtMocHelpers::MethodData<int() const>(38, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Int),
        // Method 'isPaired'
        QtMocHelpers::MethodData<bool() const>(39, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool),
        // Method 'isPairRequested'
        QtMocHelpers::MethodData<bool() const>(40, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool),
        // Method 'isPairRequestedByPeer'
        QtMocHelpers::MethodData<bool() const>(41, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool),
        // Method 'loadedPlugins'
        QtMocHelpers::MethodData<QStringList() const>(42, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QStringList),
        // Method 'hasPlugin'
        QtMocHelpers::MethodData<bool(const QString &) const>(43, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 13 },
        }}),
        // Method 'pluginsConfigFile'
        QtMocHelpers::MethodData<QString() const>(44, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Method 'setPluginEnabled'
        QtMocHelpers::MethodData<void(const QString &, bool)>(45, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 26 }, { QMetaType::Bool, 46 },
        }}),
        // Method 'isPluginEnabled'
        QtMocHelpers::MethodData<bool(const QString &) const>(47, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags, 6),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'iconName'
        QtMocHelpers::PropertyData<QString>(48, QMetaType::QString, QMC::DefaultPropertyFlags, 6),
        // property 'verificationKey'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'statusIconName'
        QtMocHelpers::PropertyData<QString>(49, QMetaType::QString, QMC::DefaultPropertyFlags, 7),
        // property 'isReachable'
        QtMocHelpers::PropertyData<bool>(50, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'reachableAddresses'
        QtMocHelpers::PropertyData<QStringList>(51, QMetaType::QStringList, QMC::DefaultPropertyFlags, 1),
        // property 'activeProviderNames'
        QtMocHelpers::PropertyData<QStringList>(52, QMetaType::QStringList, QMC::DefaultPropertyFlags, 1),
        // property 'isPaired'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'isPairRequested'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'isPairRequestedByPeer'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'pairState'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags, 3),
        // property 'supportedPlugins'
        QtMocHelpers::PropertyData<QStringList>(53, QMetaType::QStringList, QMC::DefaultPropertyFlags, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<Device, qt_meta_tag_ZN6DeviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Device::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6DeviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6DeviceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6DeviceE_t>.metaTypes,
    nullptr
} };

void Device::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Device *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pluginsChanged(); break;
        case 1: _t->linksChanged(); break;
        case 2: _t->reachableChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->pairStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->pairingFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->nameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->typeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->statusIconNameChanged(); break;
        case 8: { bool _r = _t->sendPacket((*reinterpret_cast<std::add_pointer_t<NetworkPacket&>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->requestPairing(); break;
        case 10: _t->unpair(); break;
        case 11: _t->reloadPlugins(); break;
        case 12: _t->acceptPairing(); break;
        case 13: _t->cancelPairing(); break;
        case 14: { QString _r = _t->pluginIconName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->privateReceivedPacket((*reinterpret_cast<std::add_pointer_t<NetworkPacket>>(_a[1]))); break;
        case 16: _t->linkDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 17: _t->pairingHandler_incomingPairRequest(); break;
        case 18: _t->pairingHandler_pairingFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 19: _t->pairingHandler_pairingSuccessful(); break;
        case 20: _t->pairingHandler_unpaired(); break;
        case 21: { QString _r = _t->verificationKey();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 22: { QString _r = _t->encryptionInfo();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 23: { int _r = _t->pairStateAsInt();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->isPaired();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->isPairRequested();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->isPairRequestedByPeer();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: { QStringList _r = _t->loadedPlugins();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 28: { bool _r = _t->hasPlugin((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 29: { QString _r = _t->pluginsConfigFile();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 30: _t->setPluginEnabled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 31: { bool _r = _t->isPluginEnabled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< NetworkPacket >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::pluginsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::linksChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)(bool )>(_a, &Device::reachableChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)(int )>(_a, &Device::pairStateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)(const QString & )>(_a, &Device::pairingFailed, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)(const QString & )>(_a, &Device::nameChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)(const QString & )>(_a, &Device::typeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Device::*)()>(_a, &Device::statusIconNameChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->typeAsString(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->iconName(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->verificationKey(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->statusIconName(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isReachable(); break;
        case 6: *reinterpret_cast<QStringList*>(_v) = _t->reachableAddresses(); break;
        case 7: *reinterpret_cast<QStringList*>(_v) = _t->activeProviderNames(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->isPaired(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isPairRequested(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->isPairRequestedByPeer(); break;
        case 11: *reinterpret_cast<int*>(_v) = _t->pairStateAsInt(); break;
        case 12: *reinterpret_cast<QStringList*>(_v) = _t->supportedPlugins(); break;
        default: break;
        }
    }
}

const QMetaObject *Device::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Device::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6DeviceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Device::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void Device::pluginsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Device::linksChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Device::reachableChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Device::pairStateChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Device::pairingFailed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Device::nameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Device::typeChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Device::statusIconNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
