/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was notificationinterface.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef NOTIFICATIONINTERFACE_H
#define NOTIFICATIONINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.device.notifications.notification
 */
class OrgKdeKdeconnectDeviceNotificationsNotificationInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kdeconnect.device.notifications.notification"; }

public:
    OrgKdeKdeconnectDeviceNotificationsNotificationInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDeviceNotificationsNotificationInterface();

    Q_PROPERTY(QString appName READ appName)
    inline QString appName() const
    { return qvariant_cast< QString >(property("appName")); }

    Q_PROPERTY(bool dismissable READ dismissable)
    inline bool dismissable() const
    { return qvariant_cast< bool >(property("dismissable")); }

    Q_PROPERTY(QString groupName READ groupName)
    inline QString groupName() const
    { return qvariant_cast< QString >(property("groupName")); }

    Q_PROPERTY(bool hasIcon READ hasIcon)
    inline bool hasIcon() const
    { return qvariant_cast< bool >(property("hasIcon")); }

    Q_PROPERTY(QString iconPath READ iconPath)
    inline QString iconPath() const
    { return qvariant_cast< QString >(property("iconPath")); }

    Q_PROPERTY(QString internalId READ internalId)
    inline QString internalId() const
    { return qvariant_cast< QString >(property("internalId")); }

    Q_PROPERTY(bool isConversation READ isConversation)
    inline bool isConversation() const
    { return qvariant_cast< bool >(property("isConversation")); }

    Q_PROPERTY(bool isGroupConversation READ isGroupConversation)
    inline bool isGroupConversation() const
    { return qvariant_cast< bool >(property("isGroupConversation")); }

    Q_PROPERTY(QString replyId READ replyId)
    inline QString replyId() const
    { return qvariant_cast< QString >(property("replyId")); }

    Q_PROPERTY(bool silent READ silent)
    inline bool silent() const
    { return qvariant_cast< bool >(property("silent")); }

    Q_PROPERTY(QString text READ text)
    inline QString text() const
    { return qvariant_cast< QString >(property("text")); }

    Q_PROPERTY(QString ticker READ ticker)
    inline QString ticker() const
    { return qvariant_cast< QString >(property("ticker")); }

    Q_PROPERTY(QString title READ title)
    inline QString title() const
    { return qvariant_cast< QString >(property("title")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> dismiss()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("dismiss"), argumentList);
    }

    inline QDBusPendingReply<> reply()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("reply"), argumentList);
    }

    inline QDBusPendingReply<> sendReply(const QString &message)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(message);
        return asyncCallWithArgumentList(QStringLiteral("sendReply"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void actionTriggered(const QString &key, const QString &action);
    void dismissRequested(const QString &m_internalId);
    void ready();
    void replied(const QString &message);
    void replyRequested();
};

#endif
// clazy:skip
