/****************************************************************************
** Meta object code from reading C++ file 'deviceinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "deviceinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'deviceinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN31OrgKdeKdeconnectDeviceInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgKdeKdeconnectDeviceInterface::qt_create_metaobjectdata<qt_meta_tag_ZN31OrgKdeKdeconnectDeviceInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgKdeKdeconnectDeviceInterface",
        "linksChanged",
        "",
        "nameChanged",
        "name",
        "pairStateChanged",
        "pairState",
        "pairingFailed",
        "error",
        "pluginsChanged",
        "reachableChanged",
        "reachable",
        "statusIconNameChanged",
        "typeChanged",
        "type",
        "acceptPairing",
        "QDBusPendingReply<>",
        "cancelPairing",
        "encryptionInfo",
        "QDBusPendingReply<QString>",
        "hasPlugin",
        "QDBusPendingReply<bool>",
        "isPairRequested",
        "isPairRequestedByPeer",
        "isPaired",
        "isPluginEnabled",
        "pluginName",
        "loadedPlugins",
        "QDBusPendingReply<QStringList>",
        "pairStateAsInt",
        "QDBusPendingReply<int>",
        "pluginIconName",
        "pluginsConfigFile",
        "reloadPlugins",
        "requestPairing",
        "setPluginEnabled",
        "enabled",
        "unpair",
        "verificationKey",
        "activeProviderNames",
        "iconName",
        "isReachable",
        "reachableAddresses",
        "statusIconName",
        "supportedPlugins"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'linksChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'pairStateChanged'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'pairingFailed'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'pluginsChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reachableChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'statusIconNameChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'typeChanged'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'acceptPairing'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(15, 2, QMC::AccessPublic, 0x80000000 | 16),
        // Slot 'cancelPairing'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(17, 2, QMC::AccessPublic, 0x80000000 | 16),
        // Slot 'encryptionInfo'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(18, 2, QMC::AccessPublic, 0x80000000 | 19),
        // Slot 'hasPlugin'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(20, 2, QMC::AccessPublic, 0x80000000 | 21, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'isPairRequested'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>()>(22, 2, QMC::AccessPublic, 0x80000000 | 21),
        // Slot 'isPairRequestedByPeer'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>()>(23, 2, QMC::AccessPublic, 0x80000000 | 21),
        // Slot 'isPaired'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>()>(24, 2, QMC::AccessPublic, 0x80000000 | 21),
        // Slot 'isPluginEnabled'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(25, 2, QMC::AccessPublic, 0x80000000 | 21, {{
            { QMetaType::QString, 26 },
        }}),
        // Slot 'loadedPlugins'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>()>(27, 2, QMC::AccessPublic, 0x80000000 | 28),
        // Slot 'pairStateAsInt'
        QtMocHelpers::SlotData<QDBusPendingReply<int>()>(29, 2, QMC::AccessPublic, 0x80000000 | 30),
        // Slot 'pluginIconName'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(const QString &)>(31, 2, QMC::AccessPublic, 0x80000000 | 19, {{
            { QMetaType::QString, 26 },
        }}),
        // Slot 'pluginsConfigFile'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(32, 2, QMC::AccessPublic, 0x80000000 | 19),
        // Slot 'reloadPlugins'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(33, 2, QMC::AccessPublic, 0x80000000 | 16),
        // Slot 'requestPairing'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(34, 2, QMC::AccessPublic, 0x80000000 | 16),
        // Slot 'setPluginEnabled'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, bool)>(35, 2, QMC::AccessPublic, 0x80000000 | 16, {{
            { QMetaType::QString, 26 }, { QMetaType::Bool, 36 },
        }}),
        // Slot 'unpair'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(37, 2, QMC::AccessPublic, 0x80000000 | 16),
        // Slot 'verificationKey'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(38, 2, QMC::AccessPublic, 0x80000000 | 19),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'activeProviderNames'
        QtMocHelpers::PropertyData<QStringList>(39, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'iconName'
        QtMocHelpers::PropertyData<QString>(40, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'isPairRequested'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isPairRequestedByPeer'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isPaired'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isReachable'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'pairState'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'reachableAddresses'
        QtMocHelpers::PropertyData<QStringList>(42, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'statusIconName'
        QtMocHelpers::PropertyData<QString>(43, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'supportedPlugins'
        QtMocHelpers::PropertyData<QStringList>(44, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'type'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'verificationKey'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgKdeKdeconnectDeviceInterface, qt_meta_tag_ZN31OrgKdeKdeconnectDeviceInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgKdeKdeconnectDeviceInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31OrgKdeKdeconnectDeviceInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31OrgKdeKdeconnectDeviceInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31OrgKdeKdeconnectDeviceInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgKdeKdeconnectDeviceInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgKdeKdeconnectDeviceInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->linksChanged(); break;
        case 1: _t->nameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->pairStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->pairingFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->pluginsChanged(); break;
        case 5: _t->reachableChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->statusIconNameChanged(); break;
        case 7: _t->typeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: { QDBusPendingReply<> _r = _t->acceptPairing();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<> _r = _t->cancelPairing();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<QString> _r = _t->encryptionInfo();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<bool> _r = _t->hasPlugin((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<bool> _r = _t->isPairRequested();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusPendingReply<bool> _r = _t->isPairRequestedByPeer();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 14: { QDBusPendingReply<bool> _r = _t->isPaired();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QDBusPendingReply<bool> _r = _t->isPluginEnabled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 16: { QDBusPendingReply<QStringList> _r = _t->loadedPlugins();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 17: { QDBusPendingReply<int> _r = _t->pairStateAsInt();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int>*>(_a[0]) = std::move(_r); }  break;
        case 18: { QDBusPendingReply<QString> _r = _t->pluginIconName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 19: { QDBusPendingReply<QString> _r = _t->pluginsConfigFile();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 20: { QDBusPendingReply<> _r = _t->reloadPlugins();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 21: { QDBusPendingReply<> _r = _t->requestPairing();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 22: { QDBusPendingReply<> _r = _t->setPluginEnabled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 23: { QDBusPendingReply<> _r = _t->unpair();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 24: { QDBusPendingReply<QString> _r = _t->verificationKey();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceInterface::*)()>(_a, &OrgKdeKdeconnectDeviceInterface::linksChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceInterface::*)(const QString & )>(_a, &OrgKdeKdeconnectDeviceInterface::nameChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceInterface::*)(int )>(_a, &OrgKdeKdeconnectDeviceInterface::pairStateChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceInterface::*)(const QString & )>(_a, &OrgKdeKdeconnectDeviceInterface::pairingFailed, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceInterface::*)()>(_a, &OrgKdeKdeconnectDeviceInterface::pluginsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceInterface::*)(bool )>(_a, &OrgKdeKdeconnectDeviceInterface::reachableChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceInterface::*)()>(_a, &OrgKdeKdeconnectDeviceInterface::statusIconNameChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceInterface::*)(const QString & )>(_a, &OrgKdeKdeconnectDeviceInterface::typeChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QStringList*>(_v) = _t->activeProviderNames(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->iconName(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isPairRequested(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isPairRequestedByPeer(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isPaired(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isReachable(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->pairState(); break;
        case 8: *reinterpret_cast<QStringList*>(_v) = _t->reachableAddresses(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->statusIconName(); break;
        case 10: *reinterpret_cast<QStringList*>(_v) = _t->supportedPlugins(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->type(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->verificationKey(); break;
        default: break;
        }
    }
}

const QMetaObject *OrgKdeKdeconnectDeviceInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgKdeKdeconnectDeviceInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31OrgKdeKdeconnectDeviceInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgKdeKdeconnectDeviceInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 25;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void OrgKdeKdeconnectDeviceInterface::linksChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OrgKdeKdeconnectDeviceInterface::nameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OrgKdeKdeconnectDeviceInterface::pairStateChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void OrgKdeKdeconnectDeviceInterface::pairingFailed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OrgKdeKdeconnectDeviceInterface::pluginsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OrgKdeKdeconnectDeviceInterface::reachableChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void OrgKdeKdeconnectDeviceInterface::statusIconNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OrgKdeKdeconnectDeviceInterface::typeChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
