/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was devicesftpinterface.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef DEVICESFTPINTERFACE_H
#define DEVICESFTPINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.device.sftp
 */
class OrgKdeKdeconnectDeviceSftpInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kdeconnect.device.sftp"; }

public:
    OrgKdeKdeconnectDeviceSftpInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDeviceSftpInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<QVariantMap> getDirectories()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("getDirectories"), argumentList);
    }

    inline QDBusPendingReply<QString> getMountError()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("getMountError"), argumentList);
    }

    inline QDBusPendingReply<bool> isMounted()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("isMounted"), argumentList);
    }

    inline QDBusPendingReply<> mount()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("mount"), argumentList);
    }

    inline QDBusPendingReply<bool> mountAndWait()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("mountAndWait"), argumentList);
    }

    inline QDBusPendingReply<QString> mountPoint()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("mountPoint"), argumentList);
    }

    inline QDBusPendingReply<bool> startBrowsing()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("startBrowsing"), argumentList);
    }

    inline QDBusPendingReply<> unmount()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("unmount"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void mounted();
    void unmounted();
};

#endif
// clazy:skip
