/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was virtualmonitorinterface.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef VIRTUALMONITORINTERFACE_H
#define VIRTUALMONITORINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.device.virtualmonitor
 */
class OrgKdeKdeconnectDeviceVirtualmonitorInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kdeconnect.device.virtualmonitor"; }

public:
    OrgKdeKdeconnectDeviceVirtualmonitorInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDeviceVirtualmonitorInterface();

    Q_PROPERTY(bool active READ active)
    inline bool active() const
    { return qvariant_cast< bool >(property("active")); }

    Q_PROPERTY(bool isVirtualMonitorAvailable READ isVirtualMonitorAvailable)
    inline bool isVirtualMonitorAvailable() const
    { return qvariant_cast< bool >(property("isVirtualMonitorAvailable")); }

    Q_PROPERTY(QString lastError READ lastError)
    inline QString lastError() const
    { return qvariant_cast< QString >(property("lastError")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<bool> requestVirtualMonitor()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("requestVirtualMonitor"), argumentList);
    }

    inline QDBusPendingReply<> stop()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("stop"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void activeChanged();
};

#endif
// clazy:skip
