/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was connectivityinterface.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef CONNECTIVITYINTERFACE_H
#define CONNECTIVITYINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.device.connectivity_report
 */
class OrgKdeKdeconnectDeviceConnectivity_reportInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kdeconnect.device.connectivity_report"; }

public:
    OrgKdeKdeconnectDeviceConnectivity_reportInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDeviceConnectivity_reportInterface();

    Q_PROPERTY(int cellularNetworkStrength READ cellularNetworkStrength)
    inline int cellularNetworkStrength() const
    { return qvariant_cast< int >(property("cellularNetworkStrength")); }

    Q_PROPERTY(QString cellularNetworkType READ cellularNetworkType)
    inline QString cellularNetworkType() const
    { return qvariant_cast< QString >(property("cellularNetworkType")); }

public Q_SLOTS: // METHODS
Q_SIGNALS: // SIGNALS
    void refreshed(const QString &cellularNetworkType, int cellularNetworkStrength);
};

#endif
// clazy:skip
