// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "plugin_lockdevice_debug.h"


Q_LOGGING_CATEGORY(KDECONNECT_PLUGIN_LOCKDEVICE, "kdeconnect.plugin.lockdevice", QtWarningMsg)

