/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was remotecontrolinterface.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef REMOTECONTROLINTERFACE_H
#define REMOTECONTROLINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.device.remotecontrol
 */
class OrgKdeKdeconnectDeviceRemotecontrolInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kdeconnect.device.remotecontrol"; }

public:
    OrgKdeKdeconnectDeviceRemotecontrolInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDeviceRemotecontrolInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> moveCursor(const QPoint &p)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(p);
        return asyncCallWithArgumentList(QStringLiteral("moveCursor"), argumentList);
    }

    inline QDBusPendingReply<> sendCommand(const QVariantMap &body)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(body);
        return asyncCallWithArgumentList(QStringLiteral("sendCommand"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

#endif
// clazy:skip
