/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was shareinterface.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef SHAREINTERFACE_H
#define SHAREINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.device.share
 */
class OrgKdeKdeconnectDeviceShareInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kdeconnect.device.share"; }

public:
    OrgKdeKdeconnectDeviceShareInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDeviceShareInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> openFile(const QString &file)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(file);
        return asyncCallWithArgumentList(QStringLiteral("openFile"), argumentList);
    }

    inline QDBusPendingReply<> shareText(const QString &text)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(text);
        return asyncCallWithArgumentList(QStringLiteral("shareText"), argumentList);
    }

    inline QDBusPendingReply<> shareUrl(const QString &url)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(url);
        return asyncCallWithArgumentList(QStringLiteral("shareUrl"), argumentList);
    }

    inline QDBusPendingReply<> shareUrls(const QStringList &urls)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(urls);
        return asyncCallWithArgumentList(QStringLiteral("shareUrls"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void shareReceived(const QString &url);
};

#endif
// clazy:skip
