/****************************************************************************
** Meta object code from reading C++ file 'bluetoothlinkprovider.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kdeconnect-kde-26.04.3/core/backends/bluetooth/bluetoothlinkprovider.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>
#include <QtCore/QPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bluetoothlinkprovider.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21BluetoothLinkProviderE_t {};
} // unnamed namespace

template <> constexpr inline auto BluetoothLinkProvider::qt_create_metaobjectdata<qt_meta_tag_ZN21BluetoothLinkProviderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BluetoothLinkProvider",
        "onNetworkChange",
        "",
        "onStart",
        "onStop",
        "onLinkDestroyed",
        "deviceId",
        "DeviceLink*",
        "oldPtr",
        "onStartDiscovery",
        "connectError",
        "socketDisconnected",
        "QBluetoothAddress",
        "peerAddress",
        "MultiplexChannel*",
        "socket",
        "serverNewConnection",
        "serverDataReceived",
        "QSharedPointer<MultiplexChannel>",
        "clientConnected",
        "QPointer<QBluetoothSocket>",
        "clientIdentityReceived",
        "serviceDiscovered",
        "QBluetoothServiceInfo",
        "info"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onNetworkChange'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onStart'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onStop'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onLinkDestroyed'
        QtMocHelpers::SlotData<void(const QString &, DeviceLink *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'onStartDiscovery'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'connectError'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'socketDisconnected'
        QtMocHelpers::SlotData<void(const QBluetoothAddress &, MultiplexChannel *)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Slot 'serverNewConnection'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'serverDataReceived'
        QtMocHelpers::SlotData<void(const QBluetoothAddress &, QSharedPointer<MultiplexChannel>)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 18, 15 },
        }}),
        // Slot 'clientConnected'
        QtMocHelpers::SlotData<void(QPointer<QBluetoothSocket>)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 15 },
        }}),
        // Slot 'clientIdentityReceived'
        QtMocHelpers::SlotData<void(const QBluetoothAddress &, QSharedPointer<MultiplexChannel>)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 18, 15 },
        }}),
        // Slot 'serviceDiscovered'
        QtMocHelpers::SlotData<void(const QBluetoothServiceInfo &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BluetoothLinkProvider, qt_meta_tag_ZN21BluetoothLinkProviderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BluetoothLinkProvider::staticMetaObject = { {
    QMetaObject::SuperData::link<LinkProvider::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21BluetoothLinkProviderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21BluetoothLinkProviderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21BluetoothLinkProviderE_t>.metaTypes,
    nullptr
} };

void BluetoothLinkProvider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BluetoothLinkProvider *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onNetworkChange(); break;
        case 1: _t->onStart(); break;
        case 2: _t->onStop(); break;
        case 3: _t->onLinkDestroyed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DeviceLink*>>(_a[2]))); break;
        case 4: _t->onStartDiscovery(); break;
        case 5: _t->connectError(); break;
        case 6: _t->socketDisconnected((*reinterpret_cast<std::add_pointer_t<QBluetoothAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MultiplexChannel*>>(_a[2]))); break;
        case 7: _t->serverNewConnection(); break;
        case 8: _t->serverDataReceived((*reinterpret_cast<std::add_pointer_t<QBluetoothAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSharedPointer<MultiplexChannel>>>(_a[2]))); break;
        case 9: _t->clientConnected((*reinterpret_cast<std::add_pointer_t<QPointer<QBluetoothSocket>>>(_a[1]))); break;
        case 10: _t->clientIdentityReceived((*reinterpret_cast<std::add_pointer_t<QBluetoothAddress>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSharedPointer<MultiplexChannel>>>(_a[2]))); break;
        case 11: _t->serviceDiscovered((*reinterpret_cast<std::add_pointer_t<QBluetoothServiceInfo>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DeviceLink* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothAddress >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothAddress >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPointer<QBluetoothSocket> >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothAddress >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QBluetoothServiceInfo >(); break;
            }
            break;
        }
    }
}

const QMetaObject *BluetoothLinkProvider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BluetoothLinkProvider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21BluetoothLinkProviderE_t>.strings))
        return static_cast<void*>(this);
    return LinkProvider::qt_metacast(_clname);
}

int BluetoothLinkProvider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LinkProvider::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
