/****************************************************************************
** Meta object code from reading C++ file 'notificationinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "notificationinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'notificationinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN56OrgKdeKdeconnectDeviceNotificationsNotificationInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgKdeKdeconnectDeviceNotificationsNotificationInterface::qt_create_metaobjectdata<qt_meta_tag_ZN56OrgKdeKdeconnectDeviceNotificationsNotificationInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgKdeKdeconnectDeviceNotificationsNotificationInterface",
        "actionTriggered",
        "",
        "key",
        "action",
        "dismissRequested",
        "m_internalId",
        "ready",
        "replied",
        "message",
        "replyRequested",
        "dismiss",
        "QDBusPendingReply<>",
        "reply",
        "sendReply",
        "appName",
        "dismissable",
        "groupName",
        "hasIcon",
        "iconPath",
        "internalId",
        "isConversation",
        "isGroupConversation",
        "replyId",
        "silent",
        "text",
        "ticker",
        "title"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'actionTriggered'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'dismissRequested'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'ready'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'replied'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'replyRequested'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'dismiss'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(11, 2, QMC::AccessPublic, 0x80000000 | 12),
        // Slot 'reply'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(13, 2, QMC::AccessPublic, 0x80000000 | 12),
        // Slot 'sendReply'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(14, 2, QMC::AccessPublic, 0x80000000 | 12, {{
            { QMetaType::QString, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'appName'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'dismissable'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'groupName'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'hasIcon'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'iconPath'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'internalId'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'isConversation'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isGroupConversation'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'replyId'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'silent'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'text'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'ticker'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgKdeKdeconnectDeviceNotificationsNotificationInterface, qt_meta_tag_ZN56OrgKdeKdeconnectDeviceNotificationsNotificationInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgKdeKdeconnectDeviceNotificationsNotificationInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN56OrgKdeKdeconnectDeviceNotificationsNotificationInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN56OrgKdeKdeconnectDeviceNotificationsNotificationInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN56OrgKdeKdeconnectDeviceNotificationsNotificationInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgKdeKdeconnectDeviceNotificationsNotificationInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgKdeKdeconnectDeviceNotificationsNotificationInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->actionTriggered((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->dismissRequested((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->ready(); break;
        case 3: _t->replied((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->replyRequested(); break;
        case 5: { QDBusPendingReply<> _r = _t->dismiss();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->reply();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->sendReply((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceNotificationsNotificationInterface::*)(const QString & , const QString & )>(_a, &OrgKdeKdeconnectDeviceNotificationsNotificationInterface::actionTriggered, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceNotificationsNotificationInterface::*)(const QString & )>(_a, &OrgKdeKdeconnectDeviceNotificationsNotificationInterface::dismissRequested, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceNotificationsNotificationInterface::*)()>(_a, &OrgKdeKdeconnectDeviceNotificationsNotificationInterface::ready, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceNotificationsNotificationInterface::*)(const QString & )>(_a, &OrgKdeKdeconnectDeviceNotificationsNotificationInterface::replied, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceNotificationsNotificationInterface::*)()>(_a, &OrgKdeKdeconnectDeviceNotificationsNotificationInterface::replyRequested, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->appName(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->dismissable(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->groupName(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->hasIcon(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->iconPath(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->internalId(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isConversation(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isGroupConversation(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->replyId(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->silent(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->text(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->ticker(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        default: break;
        }
    }
}

const QMetaObject *OrgKdeKdeconnectDeviceNotificationsNotificationInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgKdeKdeconnectDeviceNotificationsNotificationInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN56OrgKdeKdeconnectDeviceNotificationsNotificationInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgKdeKdeconnectDeviceNotificationsNotificationInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void OrgKdeKdeconnectDeviceNotificationsNotificationInterface::actionTriggered(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void OrgKdeKdeconnectDeviceNotificationsNotificationInterface::dismissRequested(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OrgKdeKdeconnectDeviceNotificationsNotificationInterface::ready()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OrgKdeKdeconnectDeviceNotificationsNotificationInterface::replied(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void OrgKdeKdeconnectDeviceNotificationsNotificationInterface::replyRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
