/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was devicenotificationsinterface.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef DEVICENOTIFICATIONSINTERFACE_H
#define DEVICENOTIFICATIONSINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.device.notifications
 */
class OrgKdeKdeconnectDeviceNotificationsInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kdeconnect.device.notifications"; }

public:
    OrgKdeKdeconnectDeviceNotificationsInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDeviceNotificationsInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<QStringList> activeNotifications()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("activeNotifications"), argumentList);
    }

    inline QDBusPendingReply<> sendAction(const QString &key, const QString &action)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(key) << QVariant::fromValue(action);
        return asyncCallWithArgumentList(QStringLiteral("sendAction"), argumentList);
    }

    inline QDBusPendingReply<> sendReply(const QString &replyId, const QString &message)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(replyId) << QVariant::fromValue(message);
        return asyncCallWithArgumentList(QStringLiteral("sendReply"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void allNotificationsRemoved();
    void notificationPosted(const QString &publicId);
    void notificationRemoved(const QString &publicId);
    void notificationUpdated(const QString &publicId);
};

#endif
// clazy:skip
