/********************************************************************************
** Form generated from reading UI file 'dialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIALOG_H
#define UI_DIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_Dialog
{
public:
    QVBoxLayout *verticalLayout;
    QVBoxLayout *devicePickerLayout;
    QLabel *devicePickerLabel;
    QComboBox *devicePicker;
    QVBoxLayout *urlPickerLayout;
    QLabel *urlPickerLabel;
    QHBoxLayout *buttonsLayout;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *Dialog)
    {
        if (Dialog->objectName().isEmpty())
            Dialog->setObjectName("Dialog");
        Dialog->resize(420, 142);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Dialog->sizePolicy().hasHeightForWidth());
        Dialog->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(Dialog);
        verticalLayout->setObjectName("verticalLayout");
        devicePickerLayout = new QVBoxLayout();
        devicePickerLayout->setObjectName("devicePickerLayout");
        devicePickerLabel = new QLabel(Dialog);
        devicePickerLabel->setObjectName("devicePickerLabel");

        devicePickerLayout->addWidget(devicePickerLabel);

        devicePicker = new QComboBox(Dialog);
        devicePicker->setObjectName("devicePicker");

        devicePickerLayout->addWidget(devicePicker);


        verticalLayout->addLayout(devicePickerLayout);

        urlPickerLayout = new QVBoxLayout();
        urlPickerLayout->setObjectName("urlPickerLayout");
        urlPickerLabel = new QLabel(Dialog);
        urlPickerLabel->setObjectName("urlPickerLabel");

        urlPickerLayout->addWidget(urlPickerLabel);


        verticalLayout->addLayout(urlPickerLayout);

        buttonsLayout = new QHBoxLayout();
        buttonsLayout->setObjectName("buttonsLayout");
        buttonBox = new QDialogButtonBox(Dialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Orientation::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::Cancel|QDialogButtonBox::StandardButton::Ok);

        buttonsLayout->addWidget(buttonBox);


        verticalLayout->addLayout(buttonsLayout);


        retranslateUi(Dialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, Dialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, Dialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(Dialog);
    } // setupUi

    void retranslateUi(QDialog *Dialog)
    {
        devicePickerLabel->setText(tr2i18n("Select device:", nullptr));
        urlPickerLabel->setText(tr2i18n("Enter a file location or URL:", nullptr));
        (void)Dialog;
    } // retranslateUi

};

namespace Ui {
    class Dialog: public Ui_Dialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // DIALOG_H

