// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "plugin_digitizer_debug.h"


Q_LOGGING_CATEGORY(KDECONNECT_PLUGIN_DIGITIZER, "kdeconnect.plugin.digitizer", QtWarningMsg)

