/****************************************************************************
** Meta object code from reading C++ file 'mprisremoteinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "mprisremoteinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mprisremoteinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN42OrgKdeKdeconnectDeviceMprisremoteInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgKdeKdeconnectDeviceMprisremoteInterface::qt_create_metaobjectdata<qt_meta_tag_ZN42OrgKdeKdeconnectDeviceMprisremoteInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgKdeKdeconnectDeviceMprisremoteInterface",
        "propertiesChanged",
        "",
        "requestPlayerList",
        "QDBusPendingReply<>",
        "seek",
        "offset",
        "sendAction",
        "action",
        "album",
        "artist",
        "canSeek",
        "isPlaying",
        "length",
        "localAlbumArtUrl",
        "player",
        "playerList",
        "position",
        "title",
        "volume"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'propertiesChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'requestPlayerList'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(3, 2, QMC::AccessPublic, 0x80000000 | 4),
        // Slot 'seek'
        QtMocHelpers::SlotData<QDBusPendingReply<>(int)>(5, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'sendAction'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(7, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { QMetaType::QString, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'album'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'artist'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'canSeek'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isPlaying'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'length'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'localAlbumArtUrl'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'player'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'playerList'
        QtMocHelpers::PropertyData<QStringList>(16, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'position'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'volume'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgKdeKdeconnectDeviceMprisremoteInterface, qt_meta_tag_ZN42OrgKdeKdeconnectDeviceMprisremoteInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgKdeKdeconnectDeviceMprisremoteInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42OrgKdeKdeconnectDeviceMprisremoteInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42OrgKdeKdeconnectDeviceMprisremoteInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN42OrgKdeKdeconnectDeviceMprisremoteInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgKdeKdeconnectDeviceMprisremoteInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgKdeKdeconnectDeviceMprisremoteInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->propertiesChanged(); break;
        case 1: { QDBusPendingReply<> _r = _t->requestPlayerList();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<> _r = _t->seek((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<> _r = _t->sendAction((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKdeconnectDeviceMprisremoteInterface::*)()>(_a, &OrgKdeKdeconnectDeviceMprisremoteInterface::propertiesChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->album(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->artist(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->canSeek(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isPlaying(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->length(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->localAlbumArtUrl(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->player(); break;
        case 7: *reinterpret_cast<QStringList*>(_v) = _t->playerList(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->position(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->volume(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 6: _t->setPlayer(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setPosition(*reinterpret_cast<int*>(_v)); break;
        case 10: _t->setVolume(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OrgKdeKdeconnectDeviceMprisremoteInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgKdeKdeconnectDeviceMprisremoteInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42OrgKdeKdeconnectDeviceMprisremoteInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgKdeKdeconnectDeviceMprisremoteInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void OrgKdeKdeconnectDeviceMprisremoteInterface::propertiesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
