// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "kdeconnect_conversations_debug.h"


Q_LOGGING_CATEGORY(KDECONNECT_CONVERSATIONS, "kdeconnect.conversations", QtWarningMsg)

