/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was deviceclipboardinterface.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef DEVICECLIPBOARDINTERFACE_H
#define DEVICECLIPBOARDINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.device.clipboard
 */
class OrgKdeKdeconnectDeviceClipboardInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kdeconnect.device.clipboard"; }

public:
    OrgKdeKdeconnectDeviceClipboardInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDeviceClipboardInterface();

    Q_PROPERTY(bool isAutoShareDisabled READ isAutoShareDisabled)
    inline bool isAutoShareDisabled() const
    { return qvariant_cast< bool >(property("isAutoShareDisabled")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> sendClipboard()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("sendClipboard"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void autoShareDisabledChanged(bool b);
};

#endif
// clazy:skip
