/****************************************************************************
** Meta object code from reading C++ file 'avahiserver.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "avahiserver.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'avahiserver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN35OrgFreedesktopAvahiServer2InterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopAvahiServer2Interface::qt_create_metaobjectdata<qt_meta_tag_ZN35OrgFreedesktopAvahiServer2InterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopAvahiServer2Interface",
        "StateChanged",
        "",
        "state",
        "error",
        "AddressResolverPrepare",
        "QDBusPendingReply<QDBusObjectPath>",
        "interface",
        "protocol",
        "address",
        "flags",
        "DomainBrowserPrepare",
        "domain",
        "btype",
        "EntryGroupNew",
        "GetAPIVersion",
        "QDBusPendingReply<uint>",
        "GetAlternativeHostName",
        "QDBusPendingReply<QString>",
        "name",
        "GetAlternativeServiceName",
        "GetDomainName",
        "GetHostName",
        "GetHostNameFqdn",
        "GetLocalServiceCookie",
        "GetNetworkInterfaceIndexByName",
        "QDBusPendingReply<int>",
        "GetNetworkInterfaceNameByIndex",
        "index",
        "GetState",
        "GetVersionString",
        "HostNameResolverPrepare",
        "aprotocol",
        "IsNSSSupportAvailable",
        "QDBusPendingReply<bool>",
        "RecordBrowserPrepare",
        "clazz",
        "type",
        "ResolveAddress",
        "QDBusPendingReply<int,int,int,QString,QString,uint>",
        "QDBusReply<int>",
        "int&",
        "protocol_",
        "QString&",
        "address_",
        "uint&",
        "flags_",
        "ResolveHostName",
        "QDBusPendingReply<int,int,QString,int,QString,uint>",
        "name_",
        "aprotocol_",
        "ResolveService",
        "QDBusPendingReply<int,int,QString,QString,QString,QString,int,QStr"
        "ing,ushort,QByteArrayList,uint>",
        "type_",
        "domain_",
        "host",
        "ushort&",
        "port",
        "QByteArrayList&",
        "txt",
        "ServiceBrowserPrepare",
        "ServiceResolverPrepare",
        "ServiceTypeBrowserPrepare",
        "SetHostName",
        "QDBusPendingReply<>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'StateChanged'
        QtMocHelpers::SignalData<void(int, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 4 },
        }}),
        // Slot 'AddressResolverPrepare'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(int, int, const QString &, uint)>(5, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
        }}),
        // Slot 'DomainBrowserPrepare'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(int, int, const QString &, int, uint)>(11, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 12 }, { QMetaType::Int, 13 },
            { QMetaType::UInt, 10 },
        }}),
        // Slot 'EntryGroupNew'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>()>(14, 2, QMC::AccessPublic, 0x80000000 | 6),
        // Slot 'GetAPIVersion'
        QtMocHelpers::SlotData<QDBusPendingReply<uint>()>(15, 2, QMC::AccessPublic, 0x80000000 | 16),
        // Slot 'GetAlternativeHostName'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(const QString &)>(17, 2, QMC::AccessPublic, 0x80000000 | 18, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'GetAlternativeServiceName'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(const QString &)>(20, 2, QMC::AccessPublic, 0x80000000 | 18, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'GetDomainName'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(21, 2, QMC::AccessPublic, 0x80000000 | 18),
        // Slot 'GetHostName'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(22, 2, QMC::AccessPublic, 0x80000000 | 18),
        // Slot 'GetHostNameFqdn'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(23, 2, QMC::AccessPublic, 0x80000000 | 18),
        // Slot 'GetLocalServiceCookie'
        QtMocHelpers::SlotData<QDBusPendingReply<uint>()>(24, 2, QMC::AccessPublic, 0x80000000 | 16),
        // Slot 'GetNetworkInterfaceIndexByName'
        QtMocHelpers::SlotData<QDBusPendingReply<int>(const QString &)>(25, 2, QMC::AccessPublic, 0x80000000 | 26, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'GetNetworkInterfaceNameByIndex'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(int)>(27, 2, QMC::AccessPublic, 0x80000000 | 18, {{
            { QMetaType::Int, 28 },
        }}),
        // Slot 'GetState'
        QtMocHelpers::SlotData<QDBusPendingReply<int>()>(29, 2, QMC::AccessPublic, 0x80000000 | 26),
        // Slot 'GetVersionString'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(30, 2, QMC::AccessPublic, 0x80000000 | 18),
        // Slot 'HostNameResolverPrepare'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(int, int, const QString &, int, uint)>(31, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 19 }, { QMetaType::Int, 32 },
            { QMetaType::UInt, 10 },
        }}),
        // Slot 'IsNSSSupportAvailable'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>()>(33, 2, QMC::AccessPublic, 0x80000000 | 34),
        // Slot 'RecordBrowserPrepare'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(int, int, const QString &, ushort, ushort, uint)>(35, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 19 }, { QMetaType::UShort, 36 },
            { QMetaType::UShort, 37 }, { QMetaType::UInt, 10 },
        }}),
        // Slot 'ResolveAddress'
        QtMocHelpers::SlotData<QDBusPendingReply<int,int,int,QString,QString,uint>(int, int, const QString &, uint)>(38, 2, QMC::AccessPublic, 0x80000000 | 39, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
        }}),
        // Slot 'ResolveAddress'
        QtMocHelpers::SlotData<QDBusReply<int>(int, int, const QString &, uint, int &, int &, QString &, QString &, uint &)>(38, 2, QMC::AccessPublic, 0x80000000 | 40, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { 0x80000000 | 41, 42 }, { 0x80000000 | 41, 32 }, { 0x80000000 | 43, 44 }, { 0x80000000 | 43, 19 },
            { 0x80000000 | 45, 46 },
        }}),
        // Slot 'ResolveHostName'
        QtMocHelpers::SlotData<QDBusPendingReply<int,int,QString,int,QString,uint>(int, int, const QString &, int, uint)>(47, 2, QMC::AccessPublic, 0x80000000 | 48, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 19 }, { QMetaType::Int, 32 },
            { QMetaType::UInt, 10 },
        }}),
        // Slot 'ResolveHostName'
        QtMocHelpers::SlotData<QDBusReply<int>(int, int, const QString &, int, uint, int &, QString &, int &, QString &, uint &)>(47, 2, QMC::AccessPublic, 0x80000000 | 40, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 19 }, { QMetaType::Int, 32 },
            { QMetaType::UInt, 10 }, { 0x80000000 | 41, 42 }, { 0x80000000 | 43, 49 }, { 0x80000000 | 41, 50 },
            { 0x80000000 | 43, 9 }, { 0x80000000 | 45, 46 },
        }}),
        // Slot 'ResolveService'
        QtMocHelpers::SlotData<QDBusPendingReply<int,int,QString,QString,QString,QString,int,QString,ushort,QByteArrayList,uint>(int, int, const QString &, const QString &, const QString &, int, uint)>(51, 2, QMC::AccessPublic, 0x80000000 | 52, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 19 }, { QMetaType::QString, 37 },
            { QMetaType::QString, 12 }, { QMetaType::Int, 32 }, { QMetaType::UInt, 10 },
        }}),
        // Slot 'ResolveService'
        QtMocHelpers::SlotData<QDBusReply<int>(int, int, const QString &, const QString &, const QString &, int, uint, int &, QString &, QString &, QString &, QString &, int &, QString &, ushort &, QByteArrayList &, uint &)>(51, 2, QMC::AccessPublic, 0x80000000 | 40, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 19 }, { QMetaType::QString, 37 },
            { QMetaType::QString, 12 }, { QMetaType::Int, 32 }, { QMetaType::UInt, 10 }, { 0x80000000 | 41, 42 },
            { 0x80000000 | 43, 49 }, { 0x80000000 | 43, 53 }, { 0x80000000 | 43, 54 }, { 0x80000000 | 43, 55 },
            { 0x80000000 | 41, 50 }, { 0x80000000 | 43, 9 }, { 0x80000000 | 56, 57 }, { 0x80000000 | 58, 59 },
            { 0x80000000 | 45, 46 },
        }}),
        // Slot 'ServiceBrowserPrepare'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(int, int, const QString &, const QString &, uint)>(60, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 37 }, { QMetaType::QString, 12 },
            { QMetaType::UInt, 10 },
        }}),
        // Slot 'ServiceResolverPrepare'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(int, int, const QString &, const QString &, const QString &, int, uint)>(61, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 19 }, { QMetaType::QString, 37 },
            { QMetaType::QString, 12 }, { QMetaType::Int, 32 }, { QMetaType::UInt, 10 },
        }}),
        // Slot 'ServiceTypeBrowserPrepare'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(int, int, const QString &, uint)>(62, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::QString, 12 }, { QMetaType::UInt, 10 },
        }}),
        // Slot 'SetHostName'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(63, 2, QMC::AccessPublic, 0x80000000 | 64, {{
            { QMetaType::QString, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopAvahiServer2Interface, qt_meta_tag_ZN35OrgFreedesktopAvahiServer2InterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopAvahiServer2Interface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35OrgFreedesktopAvahiServer2InterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35OrgFreedesktopAvahiServer2InterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN35OrgFreedesktopAvahiServer2InterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopAvahiServer2Interface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopAvahiServer2Interface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->StateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: { QDBusPendingReply<QDBusObjectPath> _r = _t->AddressResolverPrepare((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<QDBusObjectPath> _r = _t->DomainBrowserPrepare((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<QDBusObjectPath> _r = _t->EntryGroupNew();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<uint> _r = _t->GetAPIVersion();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<uint>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<QString> _r = _t->GetAlternativeHostName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<QString> _r = _t->GetAlternativeServiceName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<QString> _r = _t->GetDomainName();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<QString> _r = _t->GetHostName();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<QString> _r = _t->GetHostNameFqdn();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<uint> _r = _t->GetLocalServiceCookie();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<uint>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<int> _r = _t->GetNetworkInterfaceIndexByName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<QString> _r = _t->GetNetworkInterfaceNameByIndex((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusPendingReply<int> _r = _t->GetState();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int>*>(_a[0]) = std::move(_r); }  break;
        case 14: { QDBusPendingReply<QString> _r = _t->GetVersionString();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QDBusPendingReply<QDBusObjectPath> _r = _t->HostNameResolverPrepare((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 16: { QDBusPendingReply<bool> _r = _t->IsNSSSupportAvailable();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 17: { QDBusPendingReply<QDBusObjectPath> _r = _t->RecordBrowserPrepare((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<ushort>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<ushort>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[6])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 18: { QDBusPendingReply<int,int,int,QString,QString,uint> _r = _t->ResolveAddress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int,int,int,QString,QString,uint>*>(_a[0]) = std::move(_r); }  break;
        case 19: { QDBusReply<int> _r = _t->ResolveAddress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int&>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int&>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<uint&>>(_a[9])));
            if (_a[0]) *reinterpret_cast<QDBusReply<int>*>(_a[0]) = std::move(_r); }  break;
        case 20: { QDBusPendingReply<int,int,QString,int,QString,uint> _r = _t->ResolveHostName((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int,int,QString,int,QString,uint>*>(_a[0]) = std::move(_r); }  break;
        case 21: { QDBusReply<int> _r = _t->ResolveHostName((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int&>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<int&>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<uint&>>(_a[10])));
            if (_a[0]) *reinterpret_cast<QDBusReply<int>*>(_a[0]) = std::move(_r); }  break;
        case 22: { QDBusPendingReply<int,int,QString,QString,QString,QString,int,QString,ushort,QByteArrayList,uint> _r = _t->ResolveService((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int,int,QString,QString,QString,QString,int,QString,ushort,QByteArrayList,uint>*>(_a[0]) = std::move(_r); }  break;
        case 23: { QDBusReply<int> _r = _t->ResolveService((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<int&>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[11])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[12])),(*reinterpret_cast<std::add_pointer_t<int&>>(_a[13])),(*reinterpret_cast<std::add_pointer_t<QString&>>(_a[14])),(*reinterpret_cast<std::add_pointer_t<ushort&>>(_a[15])),(*reinterpret_cast<std::add_pointer_t<QByteArrayList&>>(_a[16])),(*reinterpret_cast<std::add_pointer_t<uint&>>(_a[17])));
            if (_a[0]) *reinterpret_cast<QDBusReply<int>*>(_a[0]) = std::move(_r); }  break;
        case 24: { QDBusPendingReply<QDBusObjectPath> _r = _t->ServiceBrowserPrepare((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[5])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 25: { QDBusPendingReply<QDBusObjectPath> _r = _t->ServiceResolverPrepare((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 26: { QDBusPendingReply<QDBusObjectPath> _r = _t->ServiceTypeBrowserPrepare((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 27: { QDBusPendingReply<> _r = _t->SetHostName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgFreedesktopAvahiServer2Interface::*)(int , const QString & )>(_a, &OrgFreedesktopAvahiServer2Interface::StateChanged, 0))
            return;
    }
}

const QMetaObject *OrgFreedesktopAvahiServer2Interface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopAvahiServer2Interface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35OrgFreedesktopAvahiServer2InterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopAvahiServer2Interface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void OrgFreedesktopAvahiServer2Interface::StateChanged(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
