/****************************************************************************
** Meta object code from reading C++ file 'pointerlocker.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kdeconnect-kde-26.04.3/declarativeplugin/pointerlocker.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pointerlocker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21AbstractPointerLockerE_t {};
} // unnamed namespace

template <> constexpr inline auto AbstractPointerLocker::qt_create_metaobjectdata<qt_meta_tag_ZN21AbstractPointerLockerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AbstractPointerLocker",
        "supportedChanged",
        "",
        "isSupported",
        "lockedChanged",
        "isLocked",
        "lockEffectiveChanged",
        "isLockEffective",
        "windowChanged",
        "pointerMoved",
        "QPointF",
        "delta",
        "window",
        "QWindow*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'supportedChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'lockedChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'lockEffectiveChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'windowChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pointerMoved'
        QtMocHelpers::SignalData<void(const QPointF &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'isSupported'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'isLocked'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'isLockEffective'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'window'
        QtMocHelpers::PropertyData<QWindow*>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AbstractPointerLocker, qt_meta_tag_ZN21AbstractPointerLockerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AbstractPointerLocker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21AbstractPointerLockerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21AbstractPointerLockerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21AbstractPointerLockerE_t>.metaTypes,
    nullptr
} };

void AbstractPointerLocker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AbstractPointerLocker *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->supportedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->lockedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->lockEffectiveChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->windowChanged(); break;
        case 4: _t->pointerMoved((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AbstractPointerLocker::*)(bool )>(_a, &AbstractPointerLocker::supportedChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractPointerLocker::*)(bool )>(_a, &AbstractPointerLocker::lockedChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractPointerLocker::*)(bool )>(_a, &AbstractPointerLocker::lockEffectiveChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractPointerLocker::*)()>(_a, &AbstractPointerLocker::windowChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AbstractPointerLocker::*)(const QPointF & )>(_a, &AbstractPointerLocker::pointerMoved, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWindow* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isSupported(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isLocked(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isLockEffective(); break;
        case 3: *reinterpret_cast<QWindow**>(_v) = _t->window(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setLocked(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setWindow(*reinterpret_cast<QWindow**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *AbstractPointerLocker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AbstractPointerLocker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21AbstractPointerLockerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int AbstractPointerLocker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void AbstractPointerLocker::supportedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void AbstractPointerLocker::lockedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void AbstractPointerLocker::lockEffectiveChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void AbstractPointerLocker::windowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void AbstractPointerLocker::pointerMoved(const QPointF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN15PointerLockerQtE_t {};
} // unnamed namespace

template <> constexpr inline auto PointerLockerQt::qt_create_metaobjectdata<qt_meta_tag_ZN15PointerLockerQtE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PointerLockerQt"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PointerLockerQt, qt_meta_tag_ZN15PointerLockerQtE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PointerLockerQt::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractPointerLocker::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PointerLockerQtE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PointerLockerQtE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15PointerLockerQtE_t>.metaTypes,
    nullptr
} };

void PointerLockerQt::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PointerLockerQt *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *PointerLockerQt::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PointerLockerQt::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PointerLockerQtE_t>.strings))
        return static_cast<void*>(this);
    return AbstractPointerLocker::qt_metacast(_clname);
}

int PointerLockerQt::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractPointerLocker::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
