/********************************************************************************
** Form generated from reading UI file 'sendreplydialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SENDREPLYDIALOG_H
#define UI_SENDREPLYDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>
#include "sendreplydialog.h"

QT_BEGIN_NAMESPACE

class Ui_SendReplyDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTextEdit *textView;
    SendReplyTextEdit *replyEdit;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *SendReplyDialog)
    {
        if (SendReplyDialog->objectName().isEmpty())
            SendReplyDialog->setObjectName("SendReplyDialog");
        SendReplyDialog->resize(400, 300);
        verticalLayout = new QVBoxLayout(SendReplyDialog);
        verticalLayout->setObjectName("verticalLayout");
        textView = new QTextEdit(SendReplyDialog);
        textView->setObjectName("textView");
        textView->setFocusPolicy(Qt::NoFocus);
        textView->setReadOnly(true);

        verticalLayout->addWidget(textView);

        replyEdit = new SendReplyTextEdit(SendReplyDialog);
        replyEdit->setObjectName("replyEdit");
        replyEdit->setTabChangesFocus(true);

        verticalLayout->addWidget(replyEdit);

        buttonBox = new QDialogButtonBox(SendReplyDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(SendReplyDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, SendReplyDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, SendReplyDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(SendReplyDialog);
    } // setupUi

    void retranslateUi(QDialog *SendReplyDialog)
    {
        SendReplyDialog->setWindowTitle(tr2i18n("Dialog", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SendReplyDialog: public Ui_SendReplyDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SENDREPLYDIALOG_H

