/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was remotesystemvolumeinterface.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef REMOTESYSTEMVOLUMEINTERFACE_H
#define REMOTESYSTEMVOLUMEINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.device.remotesystemvolume
 */
class OrgKdeKdeconnectDeviceRemotesystemvolumeInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kdeconnect.device.remotesystemvolume"; }

public:
    OrgKdeKdeconnectDeviceRemotesystemvolumeInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDeviceRemotesystemvolumeInterface();

    Q_PROPERTY(QString deviceId READ deviceId)
    inline QString deviceId() const
    { return qvariant_cast< QString >(property("deviceId")); }

    Q_PROPERTY(QByteArray sinks READ sinks)
    inline QByteArray sinks() const
    { return qvariant_cast< QByteArray >(property("sinks")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> sendMuted(const QString &name, bool muted)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(name) << QVariant::fromValue(muted);
        return asyncCallWithArgumentList(QStringLiteral("sendMuted"), argumentList);
    }

    inline QDBusPendingReply<> sendVolume(const QString &name, int volume)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(name) << QVariant::fromValue(volume);
        return asyncCallWithArgumentList(QStringLiteral("sendVolume"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void mutedChanged(const QString &name, bool muted);
    void sinksChanged();
    void volumeChanged(const QString &name, int volume);
};

#endif
// clazy:skip
