/****************************************************************************
** Meta object code from reading C++ file 'devicesmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kdeconnect-kde-26.04.3/models/devicesmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'devicesmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12DevicesModelE_t {};
} // unnamed namespace

template <> constexpr inline auto DevicesModel::qt_create_metaobjectdata<qt_meta_tag_ZN12DevicesModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DevicesModel",
        "rowsChanged",
        "",
        "displayFilterChanged",
        "value",
        "deviceAdded",
        "id",
        "deviceRemoved",
        "deviceUpdated",
        "refreshDeviceList",
        "receivedDeviceList",
        "QDBusPendingCallWatcher*",
        "watcher",
        "getDevice",
        "DeviceDbusInterface*",
        "row",
        "deviceForId",
        "rowForDevice",
        "displayFilter",
        "count",
        "ModelRoles",
        "NameModelRole",
        "IconModelRole",
        "StatusModelRole",
        "IdModelRole",
        "IconNameRole",
        "DeviceRole",
        "StatusFilterFlag",
        "NoFilter",
        "Paired",
        "Reachable",
        "StatusFilterFlags"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'rowsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'displayFilterChanged'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'deviceAdded'
        QtMocHelpers::SlotData<void(const QString &)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'deviceRemoved'
        QtMocHelpers::SlotData<void(const QString &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'deviceUpdated'
        QtMocHelpers::SlotData<void(const QString &)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Slot 'refreshDeviceList'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'receivedDeviceList'
        QtMocHelpers::SlotData<void(QDBusPendingCallWatcher *)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Method 'getDevice'
        QtMocHelpers::MethodData<DeviceDbusInterface *(int) const>(13, 2, QMC::AccessPublic | QMC::MethodScriptable, 0x80000000 | 14, {{
            { QMetaType::Int, 15 },
        }}),
        // Method 'deviceForId'
        QtMocHelpers::MethodData<DeviceDbusInterface *(const QString &) const>(16, 2, QMC::AccessPublic | QMC::MethodScriptable, 0x80000000 | 14, {{
            { QMetaType::QString, 6 },
        }}),
        // Method 'rowForDevice'
        QtMocHelpers::MethodData<int(const QString &) const>(17, 2, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Int, {{
            { QMetaType::QString, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'displayFilter'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'count'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags, 0),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ModelRoles'
        QtMocHelpers::EnumData<enum ModelRoles>(20, 20, QMC::EnumFlags{}).add({
            {   21, ModelRoles::NameModelRole },
            {   22, ModelRoles::IconModelRole },
            {   23, ModelRoles::StatusModelRole },
            {   24, ModelRoles::IdModelRole },
            {   25, ModelRoles::IconNameRole },
            {   26, ModelRoles::DeviceRole },
        }),
        // enum 'StatusFilterFlag'
        QtMocHelpers::EnumData<enum StatusFilterFlag>(27, 27, QMC::EnumFlags{}).add({
            {   28, StatusFilterFlag::NoFilter },
            {   29, StatusFilterFlag::Paired },
            {   30, StatusFilterFlag::Reachable },
        }),
        // flag 'StatusFilterFlags'
        QtMocHelpers::EnumData<StatusFilterFlags>(31, 27, QMC::EnumIsFlag).add({
            {   28, StatusFilterFlag::NoFilter },
            {   29, StatusFilterFlag::Paired },
            {   30, StatusFilterFlag::Reachable },
        }),
    };
    return QtMocHelpers::metaObjectData<DevicesModel, qt_meta_tag_ZN12DevicesModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DevicesModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DevicesModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DevicesModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12DevicesModelE_t>.metaTypes,
    nullptr
} };

void DevicesModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DevicesModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->rowsChanged(); break;
        case 1: _t->displayFilterChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->deviceAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->deviceRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->deviceUpdated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->refreshDeviceList(); break;
        case 6: _t->receivedDeviceList((*reinterpret_cast<std::add_pointer_t<QDBusPendingCallWatcher*>>(_a[1]))); break;
        case 7: { DeviceDbusInterface* _r = _t->getDevice((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<DeviceDbusInterface**>(_a[0]) = std::move(_r); }  break;
        case 8: { DeviceDbusInterface* _r = _t->deviceForId((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<DeviceDbusInterface**>(_a[0]) = std::move(_r); }  break;
        case 9: { int _r = _t->rowForDevice((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DevicesModel::*)()>(_a, &DevicesModel::rowsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DevicesModel::*)(int )>(_a, &DevicesModel::displayFilterChanged, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->displayFilter(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->rowCount(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDisplayFilter(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *DevicesModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DevicesModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DevicesModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int DevicesModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void DevicesModel::rowsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DevicesModel::displayFilterChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
