/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was remotecommandsinterface.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef REMOTECOMMANDSINTERFACE_H
#define REMOTECOMMANDSINTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kdeconnect.device.remotecommands
 */
class OrgKdeKdeconnectDeviceRemotecommandsInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kdeconnect.device.remotecommands"; }

public:
    OrgKdeKdeconnectDeviceRemotecommandsInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKdeconnectDeviceRemotecommandsInterface();

    Q_PROPERTY(bool canAddCommand READ canAddCommand)
    inline bool canAddCommand() const
    { return qvariant_cast< bool >(property("canAddCommand")); }

    Q_PROPERTY(QByteArray commands READ commands)
    inline QByteArray commands() const
    { return qvariant_cast< QByteArray >(property("commands")); }

    Q_PROPERTY(QString deviceId READ deviceId)
    inline QString deviceId() const
    { return qvariant_cast< QString >(property("deviceId")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> editCommands()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("editCommands"), argumentList);
    }

    inline QDBusPendingReply<> triggerCommand(const QString &key)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(key);
        return asyncCallWithArgumentList(QStringLiteral("triggerCommand"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void commandsChanged(const QByteArray &commands);
};

#endif
// clazy:skip
